define phaseOfTime (time, p, dp, ddp) {
%!%+
%\function{phaseOfTime}
%\synopsis{Compute the phase corresponding to given time}
%\usage{Double_Type[] phaseOfTime(Double_Type[] time, Double_Type, p, dp, ddp);}
%\description
%  Given a time and a period evolution by the period P \code{p},
%  first, and second derivative of P, \code{dp} and \code{ddp},
%  respectively, calculate the phase.
%
%  Note: This function does a simple taylor expansion of the phase
%  dphi = 1./P(t)dt. Time zero correpsonds to phase zero. For
%  long time arrays this function suffers from precision errors.
%
%  Since it is expected that the period changes monotonically, this
%  function throws an error if the given time is outside of the extrema
%  (if any). This only applies to non-zero \code{ddp}.
%\seealso{timeOfPhase}
%!%-
  variable a1 = 1./double(p);

  if (abs(dp)<1e-16 && abs(ddp)<1e-16)
    return time*a1;

  variable a2 = -0.5*a1*a1*dp;

  if (abs(ddp)<1e-16)
    return (time*a2+a1)*time;

  variable a3 = dp*dp*a1*a1*a1/3.-ddp*a1*a1/6.;

  if ((a2*a2-3*a3*a1>0) &&
      not all((-a2-sqrt(a2*a2-3*a3*a1))/3./a3 < time < (-a2+sqrt(a2*a2-3*a3*a1))/3./a3))
    throw DomainError, "Period evolution is invalid for the given time frame";

  return ((time*a3+a2)*time+a1)*time;
}

define timeOfPhase (phase, p, dp, ddp) {
%!%+
%\function{timeOfPhase}
%\synopsis{Compute the time corresponding to given phase}
%\usage{Double_Type[] timeOfPhase(Double_Type[] phase, Double_Type, p, dp, ddp);}
%\description
%  Given a phase and a period evolution by the period P \code{p},
%  first, and second derivative of P, \code{dp} and \code{ddp},
%  respectively, calculate the time.
%
%  Note: This function is essentially the inverse of \code{timeOfPhase}.
%  For non-zero \code{ddp} the inverse is only approximated through
%  series inversion!
%\seealso{phaseOfTime}
%!%-
  variable b1 = p;

  if (abs(dp)<1e-16 && abs(ddp)<1e-16)
    return phase*b1;

  if (abs(ddp)<1e-16)
    return p*(1.-sqrt(1.-2*dp*phase))/dp;

  % through series inversion
  variable b2 = 0.5*b1*dp;
  variable b3 = (p*dp*dp+p*p*ddp)/6.;

  if ((b2*b2-3*b3*b1>0) &&
      not all((-b2-sqrt(b2*b2-3*b3*b1))/3./b3 < time < (-b2+sqrt(b2*b2-3*b3*b1))/3./b3))
    throw DomainError, "Period evolution is invalid for the given period frame";

  return ((b3*phase+b2)*phase+b1)*phase;
}

private define periodFoldEvents (time, p) {
% relatively easy task:
%   1) sort in the events to the phase histogram `value'
%   2) calculate exposure time for each phase bin `exposure'
%     => value/exposure is the pulse profile
  variable verbose = qualifier("verbose", 1);

  % no gti given, assume all events are in a gti (edge are taken sharply)
  if (not qualifier_exists("gti") && verbose>0)
    vmessage("*** Warning: No GTIs given, assuming full time between first and last event\n  This is likely not what you want");
  variable gti = qualifier("gti", struct {start=min(time),stop=max(time)});
  variable dp = qualifier("pdot", 0.); % P dot (for Taylor approximation)
  variable ddp = qualifier("pddot", 0.); % P dot dot (also for Taylor)
  variable nbins = qualifier("nbins", 30); % number of phase bins
  variable t0 = qualifier("t0", min(gti.start)); % zero point of period evolution

  variable grid = [0:1:#nbins+1]; % overflow bin does not matter here
  variable value = UInt_Type[nbins+1];
  variable exposure = Double_Type[nbins];

  % working variables
  variable k, j, startj, n, pStart, pStop, nStart,
    nStop, inStop, inStart, tLeft, tRight,
    phase, gtiStart, gtiStop, eTime, bTime,
    gtiIdx;

  () = histogram(time, gti.start, gti.stop, &gtiIdx);

  _for k (0, length(gtiIdx)-1) {
    eTime = time[gtiIdx[k]]-t0;
    gtiStart = gti.start[k]-t0;
    gtiStop  = gti.stop[k]-t0;
    pStart = phaseOfTime(gtiStart, p, dp, ddp);
    pStop = phaseOfTime(gtiStop, p, dp, ddp);

    % get index in pulse profile of start and stop
    nStart = int(floor(pStart));
    nStop = int(floor(pStop));

    inStart = wherefirst(((pStart-nStart)-grid)<0.)-1;
    inStop = wherefirst(((pStop-nStop)-grid)<0.)-1;

    % need to go through each covered profile bin
    % and calculate the relative exposure. This is necessary
    % because if dp and ddp are non-zero, the coverage is
    % not linear.
    j = inStart;
    n = nStart;

    if ((inStart == inStop) && (nStart==nStop)) {
      % gti window is completely inside bin
      exposure[inStart] += gtiStop-gtiStart;
    } else if ((nStop - nStart) > 5) {
      % if many periods are covered in one interval it is better to loop over the phase bins only once
      if (qualifier_exists("exact"))
      {
	n = [nStart:nStop]; %nStop-nStart+1;
	_for j (0, nbins-1)
	{
	  bTime = timeOfPhase([grid[j]+n, grid[j+1]+n], p, dp, ddp);
	  exposure[j] += sum(bTime[[length(n)+1:2*length(n)-2]]-bTime[[1:length(n)-2]]);

	  if (j==inStart) % endpoints need special treatment
	    exposure[j] += (bTime[length(n)]-gtiStart);
	  if (j==inStop)
	    exposure += (gtiStop-bTime[length(n)-1]);
	}
      } else { % assume constant p over GTI such that there is a linear relation between dPhi and dT
	bTime = timeOfPhase([nStart+grid,nStop+grid], p, dp, ddp);
	exposure += (bTime[nbins+1]-bTime[nbins])/nbins;
	exposure[[inStart:nbins-1]] += (bTime[nbins]-gtiStart)/(nbins-inStart);
	exposure[[0:inStop]] += (gtiStop-bTime[nbins+1])/(inStop+1);
      }
    } else {
      bTime = timeOfPhase(n+grid, p, dp, ddp);
      startj = j;
      do {
	if (j==nbins) { % next period?
	  % phase grid change, so write it first
	  exposure[[startj:nbins-1]] += _min(gtiStop,bTime[[startj+1:nbins]])-_max(gtiStart,bTime[[startj:nbins-1]]);
	  n++;
	  bTime = timeOfPhase(n+grid, p, dp, ddp);
	  j = 0; startj = j;
	}
	j++;
      } while ((j <= inStop) || (n < nStop));
      % last partial period not covered yet, j==0 if last bin had been covered
      if (j!=0)
	exposure[[startj:j-1]] += _min(gtiStop,bTime[[startj+1:j]])-_max(gtiStart,bTime[[startj:j-1]]);
    }

    phase = phaseOfTime(eTime, p, dp, ddp);

    if (length(phase))
      value += histogram(phase-floor(phase), grid);
  }

  return struct {
    bin_lo = grid[[:-2]],
    bin_hi = grid[[1:]],
    value = 1.*value[[:-2]]/exposure, % count rate
    error = sqrt(value[[:-2]])/exposure, % count rate error
    counts = value[[:-2]],
    npts=value[[:-2]], % not sure what this is used for
    ttot = exposure % per bin exposure
  };
}

private define periodFoldLC (time, rate, rateerr, p) {
  variable verbose = qualifier("verbose", 1);

  variable dp  = qualifier("dp", 0.);
  variable ddp = qualifier("ddp", 0.);
  variable nbins = qualifier("nbins", 30);

  if (not qualifier_exists("dt") && verbose>0)
    vmessage("*** Warning: Grid spacing not given, assuming dt=diff(time)");
  variable dt = qualifier("dt"); % if not given, dt is t_k+1-t_k, can be single value
  variable fracexp = qualifier("fracexp", 1.); % if not given, full time coverage of dt is assumed
  variable t0 = qualifier("t0", min(time)); % zero point of period evolution
  variable fast = not qualifier_exists("exact");

  if (NULL == dt)
    dt = mean(time[[1:]]-time[[:-2]]);

  variable timeend, phaseLo, phaseHi;
  variable grid = [0:1:#nbins+1];
  variable value = Double_Type[nbins];
  variable exposure = Double_Type[nbins];
  variable err = Double_Type[nbins];
  variable npts;

  % working variables
  variable tStart, tStop, pStart, pStop, nStart,
    nStop, inStart, inStop, n, j, startj, k, cFracExp, cDt,
    bTime, tLeft, tRight, tVal, tTime, fastP, fastIdx;

  phaseLo = phaseOfTime(time-t0, p, dp, ddp);

  % we can speed up the calculation by alot when we ignore
  % that lc bins are finite
  if (fast) {
    fastP = phaseLo-floor(phaseLo);
    npts = histogram(fastP, grid, ,&fastIdx)[[:-2]];
    _for j (0, length(fastIdx)-2) {
      cDt = (length(dt) == 1) ? dt : dt[fastIdx[j]];
      cFracExp = (length(fracexp) == 1) ? fracexp*cDt : fracexp[fastIdx[j]]*cDt;
      value[j] = sum((rate[fastIdx[j]])*cFracExp);
      exposure[j] = sum(cFracExp);
      err[j] = (NULL != rateerr) ? sum(sqr((rateerr[fastIdx[j]])*cFracExp)) : value[j];
    }
  } else {
    timeend = time-t0+dt;
    phaseHi = phaseOfTime(timeend, p, dp, ddp);
    npts = Int_Type[nbins];

    _for k (0, length(phaseLo)-1) {
      tStart = time[k]-t0;
      % nummerical issue here, large timestamps + dt creates deviations
      tStop = tStart + ((length(dt)==1) ? dt : dt[k]);
      pStart = phaseLo[k];
      pStop = phaseHi[k];
      nStart = int(floor(pStart));
      nStop = int(floor(pStop));

      % phase bins lc bins fall into
      inStart = wherefirst(((pStart-nStart)-grid)<0)-1;
      inStop = wherefirst(((pStop-nStop)-grid)<0)-1;

      if (NULL == inStart || NULL == inStop)
	continue;

      npts[inStart]++;

      n = nStart;
      j = inStart;

      cFracExp = (length(fracexp)==1) ? fracexp : fracexp[k];

      if ((inStart == inStop) && (nStart == nStop)) {
	tTime = (tStop-tStart)*cFracExp;
	tVal = rate[k]*tTime;
	value[j] += tVal;
	exposure[j] += tTime;
	err[j] += (NULL != rateerr) ? sqr(rateerr[k]*tTime) : tVal;
      } else if ((nStop - nStart) > 5) {
	% if many periods are covered in one interval it is better to loop over the phase bins only once
	% and for the LC it is safe to assume the period does not change significantly over one bin
	bTime = timeOfPhase([nStart+grid,nStop+grid], p, dp, ddp);
	exposure += (bTime[nbins+1]-bTime[nbins])/nbins;
	exposure[[inStart:nbins-1]] += (bTime[nbins]-tStart)/(nbins-inStart);
	exposure[[0:inStop]] += (tStop-bTime[nbins+1])/(inStop+1);
      } else {
	% otherwise we loop through the phases until we hit the interval limit
	bTime = timeOfPhase(n+grid, p, dp, ddp);
	startj = j;
	do {
	  if (j == nbins) {
	    tTime = (_min(tStop,bTime[[startj+1:nbins]])-_max(tStart,bTime[[startj:nbins-1]]))*cFracExp;
	    tVal = rate[k]*tTime;
	    exposure[[startj:nbins-1]] += tTime;
	    value[[startj:nbins-1]] += tVal;
	    err[[startj:nbins-1]] += (NULL != rateerr) ? rateerr[k]*tTime : tVal;
	    n++;
	    bTime = timeOfPhase(n+grid, p, dp, ddp);
	    j=0; startj = j;
	  }
	  j++;
	} while ((j <= inStop) || (n < nStop));
	% last coverage might not be handled
	if (j!=0) {
	  tTime = (_min(tStop,bTime[[startj+1:j]])-_max(tStart,bTime[[startj:j-1]]))*cFracExp;
	  tVal = rate[k]*tTime;
	  exposure[[startj:j-1]] += tTime;
	  value[[startj:j-1]] += tVal;
	  err[[startj:j-1]] += (NULL != rateerr) ? sqr(rateerr[k]*tTime) : tVal;
	}
      }
    }
  }

  return struct {
    bin_lo = grid[[:-2]],
    bin_hi = grid[[1:]],
    value = 1.*value/exposure, % count rate
    error = sqrt(err)/exposure, % count rate error
    counts = value, % counts
    npts=npts, % how often a bin was hit by start time (not sure what this is used for)
    ttot = exposure, % per bin exposure
  };
}

%%%%%%%%%%%%%%%%%%%%%%%%
define pfold ()
%%%%%%%%%%%%%%%%%%%%%%%%
%!%+
%\function{pfold}
%\synopsis{folds a lightcurve or event list on a given period}
%\usage{Struct_Type pp = pfold(Double_Type t, r, p);
%\altusage{Struct_Type pp = pfoldpfold(Double_Type t, r, p, e);}
%\altusage{Struct_Type pp = pfold(Double_Type t, p);}}
%\qualifiers{
%\qualifier{nbins}{[=30] number of bins for the pulse profile}
%\qualifier{exact}{take finite lightcurve bins into account (LC case).
%             take differantial phase grid into account (Events case).}
%\qualifier{dt}{lightcurve bin size.}
%\qualifier{fracexp}{fractional exposure per lightcurve bin.}
%\qualifier{t0}{set reference time.}
%\qualifier{pdot}{[=0] first derivative of pulse period p.}
%\qualifier{pddot}{[=0] second derivative of pulse period p.}
%\qualifier{gti}{GTIs for event data, given as struct{start=Double_Type, stop=Double_Type}}
%}
%\description
%   Calculates the pulse profile of a given lightcurve or event list
%   for times \code{t} and rate \code{r}. In case of an event list
%   normally the use of GTIs is necessary to ensure correct results.
%
%   If the qualifiers \code{pdot} or \code{pddot} are given and not
%   zero a taylor expansion of the period evolution is used to
%   calculate the mapping to phase (from \code{t0}). If \code{pddot}
%   is non-zero the result is only approximately correct. See
%   \code{phaseOfTime}.
%
%   The input arrays \code{t}, \code{r}, and \code{e} can also be
%   given as qualifiers.
%
%   If the \code{exact} qualifier is given the finite size of lightcurve
%   bins is taken into account by linear interpolation. However, the
%   computation time is increased. For event lists, the code will take the
%   differentail phase grid change into account. Otherwise the exposure
%   time is approximated assuming a constant period over one GTI frame.
%
%   The returned structure contains the fields bin_lo, bin_hi, value
%   and err, ready for plotting the pulse profile (as rate). For further
%   diagnostics the \code{counts} (counts) and \code{ttot} (per-bin exposure)
%   are given. The field \code{npts} contains the number of lightcurve bins
%   starting in each bin.
%\seealso{phaseOfTime, timeOfPhase}
%!%-
{
  variable chatty = qualifier("chatty",qualifier("verbose", _Isisscripts_Verbose)) ;
  variable t,r = NULL,e = NULL,p;

  switch(_NARGS)
  {case 2:
    (t,p) = ();
    (r,e) = (qualifier("r", r), qualifier("e", e));
  }
  {case 3:
    (t,r,p) = ();
    e = qualifier("e", e);
  }
  {case 4:
    (t,r,p,e) = ();
  }
  {() = (); help(_function_name()); return; }

  if (NULL == r) {
    % event data
    return periodFoldEvents(t, p;; __qualifiers);
  } else {
    % lightcurve
    if (length(t) != length(r) || (e != NULL && length(t)!=length(e)))
      throw UsageError, "Time and rate (or error) arrays are not of equal length";
    return periodFoldLC(t, r, e, p;; __qualifiers);
  }
}
