%%%%%%%%%%%%%
define CCF_1d()
%%%%%%%%%%%%%
%!%+
%\function{CCF_1d}
%\synopsis{calculates the cross correlation function of two (1d) arrays}
%\usage{Double_Type CCF = CCF_1d(a1, a2, Integer_Type dx);
%\altusage{Array_Type CCF = CCF_1d(a1, a2);}
%}
%\qualifiers{
%\qualifier{notperiodic}{arrays are not extended periodically.}
%}
%\description
%    The arrays \code{a1} and \code{a2} have to have the same length \code{n}.
%    They can be passed directly or as references to arrays.
%
%    By default (unless the \code{notperiodic} qualifier is given), both arrays
%    are considered to be extended with periodic boundary conditions.
%    When using \code{notperiodic}, chose \code{|dx| < n-1}, since the larger \code{|dx|},
%    the smaller are the parts of the arrays that can actually be compared.
%
%    This function computes the correlation of \code{a1[x]} and \code{a2[x-dx]}:
%       \code{CCF = sum_x { (a1[x]-<a1>)/|a1| * (a2[x-dx]-<a2>)/|a2| }}
%    For periodic boundary conditions, the entire array is considered,
%    else the index \code{x} takes only values such that \code{x} and \code{x-dx} are contained.
%    The norm \code{|a| = sqrt{ sum_x a[x]^2 }} ensures that for any array \code{a},
%       \code{CCF_1d(a, a, 0) = +1}   and   \code{CCF_1d(a, -a, 0) = -1}.\n
%    \n
%    If only the two arrays \code{a1} and \code{a2} are given, the function returns
%    an array with all possible correlations between them, i.e.:
%    \code{[ CCF_1d(a1, a2, 0), CCF_1d(a1, a2, 1), ..., CCF_1d(a1, a2, n-1) ]}
%    This is most reasonable for periodic boundary conditions.
%\notes
%    The function is vectorized on \code{dx}, i.e., can handle an array \code{dx}.
%\example
%    variable a1 = Double_Type[n];  % some data
%    variable a2 = Double_Type[n];  % some other data
%
%    variable dx = [-n/2 : n/2];
%    variable CCF_np = CCF_1d(a1, a2, dx; notperiodic);
%
%    variable CCF_p = CCF_1d(a1, a2);  % This is equivalent to:
%    variable CCF_p = array_map(Double_Type, &CCF_1d, &a1, &a2, [0:n-1]);
%\seealso{CCF_2d}
%!%-
{
   variable arr1, arr2, dxs = NULL, periodic = not qualifier_exists("notperiodic");
   switch(_NARGS)
   { case 3: (arr1, arr2, dxs) = (); }
   { case 2: (arr1, arr2) = (); }
   { return help(_function_name()); }

   if (typeof(arr1) == Ref_Type)  arr1 = @arr1;
   if (typeof(arr2) == Ref_Type)  arr2 = @arr2;
   variable n = length(arr1);
   if (length(arr2) != n)  throw InvalidParmError;

   dxs = dxs == NULL ? [0 : n-1] : int(dxs);
   if (periodic)  dxs = ((dxs mod n) + n) mod n;  % => 0 <= dxs < n

   variable X1 = [0 : n-1];
   variable res = Double_Type[length(dxs)];
   variable dx, i = 0;
   foreach dx (dxs) {
      ifnot (periodic) {
         X1 = [int(_max(0,dx)) : n+int(_min(0,dx))-1];
      }
      variable a1 = arr1[X1   ];  a1 -= mean(a1);  a1 /= sqrt(sumsq(a1));
      variable a2 = arr2[X1-dx];  a2 -= mean(a2);  a2 /= sqrt(sumsq(a2));
      res[i] = sum(a1*a2);
      i++;
   }

   return typeof(dxs) != Array_Type ? res[0] : res;
}
