define array_permutation_matrix()
%!%+
%\function{array_permutation_matrix}
%\synopsis{returns all possible permutations of the given arrays}
%\usage{Array_Type array_permutation_matrix(Array_Type[] or List_Type arrays);}
%\description
%    Calculates the index arrays of all possible permutations of the
%    elements in the given arrays (see the below example). The input
%    has to be either an array of arrays (Array_Type) or a list of
%    arrays (List_Type). The returned matrix (J,K) is an array of
%    integer arrays. The element (J,K) is the index of the K^th input
%    array, which corresponds to the J^th permutation.    
%\example
%    variable arr = Array_Type[3];
%    arr[0] = [10,20,30,40];
%    arr[1] = [50,60];
%    arr[2] = [70,80,90];
%    
%    variable matrix = array_permutation_matrix(arr);
%    % Returns an Array_Type[24] corresponding to 4*2*3 = 24 possible
%    % permutations. Each element is an Integer_Type[3] containing the
%    % indices for each of the 3 input arrays:
%    % matrix[0]  = [0,0,0] -> [10,50,70]
%    % matrix[1]  = [1,0,0] -> [20,50,70]
%    % matrix[2]  = [2,0,0] -> [30,50,70]
%    % matrix[3]  = [3,0,0] -> [40,50,70]
%    % matrix[4]  = [0,1,0] -> [10,60,70]
%    % matrix[5]  = [1,1,0] -> [20,60,70]
%    % ...
%    % matrix[23] = [3,1,2] -> [40,60,90]
%!%-
{
  variable arr;
  switch (_NARGS)
    { case 1: (arr) = (); }
    { help(_function_name); }

  % sanity checks
  if (typeof(arr) == List_Type) {
    arr = list_to_array(arr);
  } else if (typeof(arr) != Array_Type) {
    vmessage("error (%s): input has to be of Array_Type or List_Type", _function_name);
  }
  
  % get array dimensions and number of permutations
  variable dim = array_map(Array_Type, &array_shape, arr);
  variable num = int(outer_product(__push_array(dim))[*][0]); 
  
  % initialize matrix
  variable matrix = Array_Type[num];
  matrix[0] = Integer_Type[length(dim)]; % start with first period indices

  % loop over permutations
  variable n, d;
  _for n (1, num-1, 1) {
    % start with previous permutation
    matrix[n] = @(matrix[n-1]);
    % increment
    d = 0;
    forever {
      matrix[n][d]++;
      if (matrix[n][d] == dim[d][0]) { matrix[n][d] = 0; d++; }
      else { break; }
    }
  }

  return matrix;
}
