define associate_coords(ra1, dec1, ra2, dec2)
%!%+
%\function{associate_coords}
%\synopsis{computes the smallest pairwise angular separation between arrays of positions and returns the indices}
%\usage{Struct_Type result = associate_coords(Double_Type[n1] RA1, Double_Type[n1] DEC1),Double_Type[n2] RA2, Double_Type[n2] DEC2);}
%\qualifiers{
%\qualifier{nr_closest}{[=1]      set to N, to obtain the N closest elements.
%                            In this case additionally the index of the second, ..., N-th closest element is
%                            returned as \code{result.index2[*,0], ..., result.index2[*,N-2]},
%                            the same holds for \code{result.sep2}.}
%\qualifier{unit}{[\code{="deg"}]        unit of the angular coordinates}
%\qualifier{alpha1_unit}{[\code{="deg"}] unit of the RA1}
%\qualifier{alpha2_unit}{[\code{="deg"}] unit of the RA2}
%\qualifier{delta1_unit}{[\code{="deg"}] unit of the DEC1}
%\qualifier{delta2_unit}{[\code{="deg"}] unit of the DEC2}
%}
%\description
%    For each position \code{(RA1[i], DEC1[i])} this function searches the element \code{(RA2[j],DEC2[j])}
%    with the smallest difference to \code{(RA1[i],DEC[i])}. It returns the indices \code{j}
%    as \code{result.index[i]} and the corresponding separation in rad as \code{result.sep[i]}.
%    The units of each coordinate can be set with the qualifier \code{unit}, but also independently,
%    as for the function \code{greatcircle_distance}.
%\seealso{greatcircle_distance, associate_arrays}
%!%-
{
  variable q = __qualifiers();
  variable nc = qualifier("nr_closest", 1);

  variable result = struct {index, sep};
  variable i,j,d,k;
  result.index =  Integer_Type [length(ra1), nc];
  result.sep  =  Double_Type  [length(ra1), nc];
  result.sep[*,*] = DOUBLE_MAX;

  for (i=0; i<length(ra1); i++)
  {
    for (j=0; j<length(ra2); j++)
    {
      d = greatcircle_distance(ra1[i], dec1[i], ra2[j], dec2[j];; q);

      k = nc-1;
      while ( k >= 0 && abs(d) < abs(result.sep[i,k]) )
        {
          if (k < nc-1)
          {
            result.sep[i,k+1] = result.sep[i,k];
            result.index[i,k+1] = result.index[i,k];
          }
          result.sep[i,k] = d;
          result.index[i,k] = j;
          k--;
        }
    }
  }
  if (nc > 1)
  {
    result = struct_combine(result,"index2","sep2");
    result.index2 = result.index[*,[1:]];
    result.sep2 = result.sep[*,[1:]];
  }
  result.index = result.index[*,0];
  result.sep = result.sep[*,0];
  return result;
}
