%%%%%%%%%%%%%%%%%%%%%
define BinaryPos()
%%%%%%%%%%%%%%%%%%%%%
%!%+
%\function{BinaryPos}
%\synopsis{computes the position of a star in a binary system}
%\usage{Double_Type[] BinaryPos(Double_Type[] time)}
%\qualifiers{
%\qualifier{porb}{Orbital period at the epoch [same units as time argument], mandatory}
%\qualifier{eccentricity}{Eccentricity, (0<=e<1)}
%\qualifier{omega}{Longitude of periastron [degrees], mandatory}
%\qualifier{asini}{Projected semi-major axis (a sin i), mandatory for spectroscopic binaries, 
%                    give it in light seconds (a sini/c) if you want to compute a time correction}
%\qualifier{semi}{semimajor axis, mandatory for visual binaries}
%\qualifier{bigomega}{Longitude of the ascending node [degrees], mandatory for visual binaries}
%\qualifier{incl}{inclination [degrees], mandatory for visual binaries}
%\qualifier{t0}{epoch for mean longitude of 0 degrees (periastron, MJD)}
%\qualifier{t90}{epoch for mean longitude of 90 degrees (MJD)}
%\qualifier{pporb}{rate of change of the orbital period (s/s) (default 0)}
%}
%\description
%    (transcribed from IDL-programm BinaryPos.pro)
%
%    For spectroscopic binaries, the z-position of the emitting object
%    is computed (negative: closer to Earth). In case of visual binaries
%    the output is a 3d array containing the xyz position of the object
%    in a right handed coordinate system where the z axis points away from
%    Earth, the x axis points towards the North Celestial Pole, and where
%    the y-axis increases towards East.
%
%    Follows equations from Hilditch's book.
%
%    Qualifiers t90 and t0 have to be in days and in the same time
%    system as time (e.g. JD or MJD)
%
%    Circular orbits:
%         * if time of lower conjunction Tlow is known, set
%           t0=Tlow and omega=0
%         * if time of ascending node is known, Tasc, set
%           t90=Tasc and omega=0
%         * if time of mid eclipse is known, Tecl, set
%           t0=Tecl-0.25*porb and omega=0
%!%-
{
  switch(_NARGS)
  { case 1: variable t = (); }
  { help(_function_name()); return; }
  % make sure that time is an array
  t = [t];
 
  if (not qualifier_exists("porb") or not qualifier_exists("eccentricity")
      or (not qualifier_exists("t90") and not qualifier_exists("t0")))
     throw RunTimeError, "Need porb, eccentricity, and omega values, and one of t90 or t0";

  if (qualifier_exists("asini") + qualifier_exists("semi") != 1)
     throw RunTimeError, "Need exactly one of asini and semi";

  if (qualifier_exists("semi"))
  {
     if (not qualifier_exists("bigomega") or not qualifier_exists("incl"))
       throw RunTimeError, "Visual binaries: Need bigomega and incl";
  }

  if (qualifier_exists("t90") and qualifier_exists("t0"))
     throw RunTimeError, "Only one of the t90 and t0 arguments is allowed";

  if (qualifier("eccentricity",0.0) != 0.0 and not qualifier_exists("omega"))
     throw RunTimeError, "Need omega keyword for noncircular orbits";

  if (qualifier("eccentricity",0.0) < 0.0) 
     throw RunTimeError, "Eccentricity must be positive";

  if (qualifier("eccentricity",0.0) >= 1.0) 
     throw RunTimeError, "Binarypos only works for 0<=ecc<1";

  % qualifiers
  variable porb = qualifier("porb");
  variable ecc = qualifier("eccentricity",0.0);
  variable omega = qualifier("omega");
  variable asini = qualifier("asini");
  variable semi = qualifier("semi");
  variable bigomega = qualifier("bigomega");
  variable incl = qualifier("incl");
  variable t90 = qualifier("t90");
  variable pporb = qualifier("pporb",0.0);
  % compute t0 from t90 if necessary
  % note that this assumes pporb<<porb, which can be generally assumed
  variable t0 = qualifier("t0");
  if (t0 == NULL) t0 = qualifier("t0", t90+(omega-90.)/360. * porb);

  variable theta, pos;
  variable tper = (t-t0) / porb;
  variable m = 2.0 * PI * (tper * (1.0 - 0.5 * pporb * tper));
  variable eanom = KeplerEquation(m,ecc);

  % spectroscopic binaries
  if (asini != NULL)
  {
    omega = omega * PI / 180.0;
    variable sinw = sin(omega);
    variable cosw = cos(omega);
    variable sq = sqrt((1.0 - ecc)*(1.0 + ecc));
    variable sin_e, cos_e;
    
%%    pos = Double_Type[t];

    sin_e = sin(eanom);
    cos_e = cos(eanom);

    pos = asini * (sinw * (cos_e - ecc) + sq * cosw * sin_e);
%%    theta = 2.0 * atan(sqrt ((1.0 + ecc) / (1.0 - ecc)) * tan(eanom / 2.0));
%%    pos = asini * (1.0 - ecc^2.0) * sin(theta + omega) / (1.0 + ecc * cos(theta));
  }
  % visual binaries
  else
  {
    bigomega = bigomega * PI / 180.0;
    incl = incl * PI / 180.0;
    variable r, sin_to, cos_to;
    variable sin_o = sin(bigomega);
    variable cos_o = cos(bigomega);
    variable sin_i = sin(incl);
    variable cos_i = cos(incl);

    pos = Double_Type[3,length(t)];

    % true anomaly
    theta = 2.0 * atan(sqrt ((1.0 + ecc) / (1.0 - ecc)) * tan(eanom / 2.0));
    % radius vector
    r = semi * (1.0 - ecc * cos(eanom));
    % helpers
    sin_to = sin(theta + omega);
    cos_to = cos(theta + omega);
    % Hilditch, eq. 2.61
    pos[0,*] = r * (cos_o * cos_to - sin_o * sin_to * cos_i);
    pos[1,*] = r * (sin_o * cos_to + cos_o * sin_to * cos_i);
    pos[2,*] = r * sin_to * sin_i;
  }

  return pos;
}
