%%%%%%%%%%%%%%%%%%%%%
define GhoshLamb79()
%%%%%%%%%%%%%%%%%%%%%
%!%+
%\function{GhoshLamb79}
%\synopsis{calculates the spin-up of a neutron star after Ghosh & Lamb (1979)}
%\usage{Double_Type GhoshLamb79(Double_Type pulse_period; disk);
% or Double_Type GhoshLamb79(Double_Type pulse_period; wind);}
%\qualifiers{
%    \qualifier{fastness}{hook into the issue of a fastness parameter >=0.9. If
%               this qualifier is set to "limit" then the fastness is
%               set to 0.9 in this case. If set to a reference
%               function (Ref_Type) then a hook function of the form
%                 Double_Type fastness_hook(fastness, P, L, M, R, B)
%               is called, which is given the computed 'fastness' and
%               all neutrons star parameters. It has to return the new
%               value for the fastness.}
%    \qualifier{neutron star parameters:}{The qualifiers L, M, R, B, mu, V0,
%      cs, Vorb, a, xi (see the description for details)}
%}
%\description
%    This implements the accretion torque theory by
%      Ghosh & Lamb, 1979, ApJ 234, 296
%    (here after GL79) which calculates the spin-up of a neutron star
%    spinning with a period of 'pulse_period' (given in seconds). The
%    returned Pdot will be in seconds per seconds.
%
%    The theory depends on the physical parameters of the neutron star
%    and the accretion mode. All of these are controlled by qualifiers.
%
%    Two accretion modes are available (set as corresponding qualifier):
%      disk - accretion from an accretion disk (Eq. 15 of GL79)
%      wind - accretion from a stellar wind (Eq. 24 of GL79)
%
%    Three neutron star parameters are used independently from the mode:
%      L - the X-ray luminosity (in 10^37 erg/s; default: 1)
%      M - the neutron star mass (in solar masses; default: 1.4)
%      R - the neutron star radius (in km; default: 11.5)
%
%    For the disk-mode the magnetic field strength is needed. One of
%    the following quantities has to be given:
%      mu - magnetic moment (in 10^30 G cm^3; default: 1)
%      B  - surface magnetic field strength (in 10^12 G; overwrites mu)
%    For the wind-mode several parameters are needed:
%      V0   - the capture velocity (see Eq. 20 of GL79) (in km/s;
%             default: 1000)
%      cs   - the sound speed at the capture radius (in km/s; default: 0)
%      Vorb - the orbital velocity (in km/s; default related to:
%             V0^2 = Vorb^2 + Vwind^2 + cs^2 assuming Vwind = Vorb)
%      a    - the binary separation (in lt-s; default: 100)
%      xi   - "on the order of unity" (see Eq. 22 of GL79; default: 1)
%
%    The function should be vectorized, i.e., an array of pulse
%    periods can be given or the qualifiers can be arrays.
%
%    In case of disk accretion, the so-called fastness parameter is
%    computed after Eq. 16 of GL79. This equation is accurate by 5% for
%    fastnesses <0.9. For larger values a warning message is displayed.
%    For fastnesses >=1.0 the solution is a complex number and therefore
%    a _NaN value is returned and an error message is displayed. The
%    'fastness' qualifier can be used to hook into this issue.
%\seealso{pulsarGL79}
%!%-
{
  % pulse period
  if (_NARGS != 1) { help(_function_name); return; }
  variable P = ();
    
  % accretion type (wind vs. disk)
  variable type;
  if (qualifier_exists("disk")) { type = 1; }
  else if (qualifier_exists("wind")) { type = 2; }
  else {
    help(_function_name);
    return;
  }

  % neutron star parameters
  variable L = qualifier("L", 1.); % in 1e37 erg/s
  variable M = qualifier("M", 1.4);  % in solar masses
  variable R = .1 * qualifier("R", 11.5); % in 1e6 cm

  % moment of intertia
  variable I = 2./5 * (M*1.989) * R^2; % in 1e45 g cm^2

  % disk accretion
  if (type == 1) {
    % magnetic moment
    variable mu = qualifier("mu", 1.);
    if (qualifier_exists("B")) {
      variable B = qualifier("B"); % in 1e12 G
      mu = .5 * B * R^3; % in 1e30 G/cm^3
    }

    % first structure function (eq. 17)
    variable S1 = R^(6./7)*M^(-3./7)/I;

    % second structure function (eq. 18)
    variable S2 = R^(-3./7)*M^(-2./7);

    % fastness parameter (eq. 16)
    variable omegas = 1.35*mu^(6./7)*S2*(P*L^(3./7))^(-1);
    if (any(omegas >= 0.9)) {
      % hook
      variable fastness = qualifier("fastness", NULL);
      if (typeof(fastness) == Ref_Type) {
	omegas = (@fastness)(omegas, P, L, M, R, mu);
      }
      else if (typeof(fastness) == String_Type && fastness == "limit") {
	omegas = _min(omegas, 0.9);
      }
      % print warnings/errors
      else {
	if (any(omegas >= 1.0)) {
	  vmessage("error (%s): fastness parameter >= 1.0", _function_name);
	} else {
	  vmessage("warning (%s): fastness parameter >= 0.9", _function_name);
	}
      }
    }
    
    % accretion torque (eq. 10)
    variable n = 1.39*(1.-omegas*(4.03*(1-omegas)^0.173-0.878))/(1.-omegas);

    % spin up (eq. 15)
    return -5e-5*mu^(2./7)*n*S1*(P*L^(3./7))^2 / 365/86400;
  }

  % wind accretion
  if (type == 2) {
    variable V0 = qualifier("V0", 1000); % in km/s
    variable xi = qualifier("xi", 1.); % "on the order of unity" (GL79)
    % capture velocity (text after eq. 20): V^2 = Vorb^2 + Vwind^2 + cs^2
    variable cs = qualifier("cs", 0); % sound speed at capture radius
    % assuming that Vorb and Vwind are equal:
    variable vorb = qualifier("Vorb", sqrt(.5*(V0^2-cs^2))); % in km/s
    variable a = qualifier("a", 100)*(Const_c*1e-10); % in 10^10 cm

    % capture radius (eq. 22)
    variable ra = xi*2*(Const_G*1e8)*(M*1.989)/V0^2*1e5; % in 10^10 cm

    % specific angular moment (eq. 21)
    variable la = .5*vorb*ra^2/a * 1e-2; % in 1e17 cm^2/s

    % spin up (eq. 24)
    return -3.8e-5/R/I*la*P^2*L / 365/86400;
  }
}
