require( "vector" );

%%%%%%%%%%%%%%%%%%%%%%%%
define Roche_lobe_surface()
%%%%%%%%%%%%%%%%%%%%%%%%
%!%+
%\function{Roche_lobe_surface}
%\synopsis{calculates surface elements of a binary star}
%\usage{Struct_Type = Roche_lobe_surface(Double_Type mq, Double_Type fill, Integer_Type ntheta);}
%\qualifiers{
%  \qualifier{noRoche}{neglect the Roche potential -> spherical star}
%  \qualifier{noNorm}{do not return the normal vectors}
%  \qualifier{noArea}{do not return the areas}
%  \qualifier{poly}{return the polygons defining the elements}
%  \qualifier{rotate}{rotate the star around its (z-)axis by the
%            given degrees (default: 0)}
%  \qualifier{eps}{numerical precision (default: 1e-8)}
%  \qualifier{chatty}{be chatty}
%}
%\description
%  This function calculates the surface of the primary star
%  in a binary, which is deformed by the Roche potential.
%  Since this potential can not be solved analytically the
%  surface must be divided into elements. The number of
%  elements into theta direction can be specified by the
%  'ntheta' argument. The total number of surface elements
%  taking into account the singularity at the poles and the
%  symmetry in theta is here
%    2*ntheta * (ntheta-2) + 2
%  The Roche potential for a spherical orbit depends on the
%  mass ratio mq = M_primary / M_secondary
%  To determine the size of the star, the undeformed radius
%  Rz in direction of the angular velocity vector (omega)
%  can be used via the so-called fill factor:
%    fill = Rz / R_crit
%  where R_crit is the critical undeformed radius for a
%  star, which fills its Roche lobe completely. In that
%  case, the potential value at the surface is equal to
%  that at the first Lagrange point, which might lead to
%  mass transfer onto the companion (Roche lobe overflow).
%
%  The returned structure contains the following fields:
%    Vector_Type[] r - position vector to each element
%                      starting at the stars center
%    Vector_Type[] n - normal vector of each element
%    Vector_Type[] A - area of each element
%    Vector_Type[] p - if the 'poly' qualifier is given,
%                      it contains the polygons defining
%                      each surface element
%
%  NOTE: The origin of the reference frame is in the center
%        of the star and the first Lagrange point is on
%        the x-axis, while the rotation axis of the star is
%        along the z-axis!
%        If you want to rotate the star around its rotation
%        axis (z-axis), use the 'rotate'-qualifier!
%\seealso{Roche_critical, Roche_potential, radius_to_unit, Vector_Type}
%!%-
{
 variable mq, fill, ntheta;
 switch(_NARGS)
   { case 3: (mq, fill, ntheta) = (); mq = 1./mq; }
   { help(_function_name()); return; }

 if (qualifier_exists("chatty")) vmessage("initializing", _function_name);
 % surface polygons expressed in spherical coordinates
 variable pphi    = 2.*PI*[0:2*ntheta-2]/(2*ntheta-2); % phi (one more for phi=2PI)
 variable ptheta  = 1.*PI*[0:ntheta-1]/(ntheta-1);     % theta
 variable pradius = Double_Type[2*ntheta-1,ntheta];    % radius as function of [phi, theta]

 % convert fill factor into radius in units of binary displacement
 variable radius = radius_to_unit(fill, "fill", "disp"; mq = mq);
  
 if (qualifier_exists("chatty")) vmessage("calculating radii", _function_name);
 % calculate radii
 if (qualifier_exists("noRoche")) pradius = pradius + radius;
 % by numerically solve Roche potential for given radius
 else
 {
   % critical potential on surface
   variable surfpot = Roche_potential(0.0, 0.0, radius; q = mq);
   variable p, t;
   _for p (0, 2*ntheta-2, 1)
   {
     _for t (0, ntheta-1, 1)
     {
       % singularity on poles and meridian
       if (t == 0 && p > 0) pradius[p,t] = pradius[0,0]; % theta=0
       else if (t == ntheta-1 && p > 0) pradius[p,t] = pradius[0,ntheta-1]; % theta=PI
       else if (p == 2*ntheta-2) pradius[p,t] = pradius[0,t]; % meridian
       % find radius corresponding to potential value
       else
       {
         variable tmprad = 0., dr = .1, pot = -_Inf, tmppot, j = 0;
         variable eps = qualifier("eps", 1e-8);
	 % start at radius 0 (=infinite negative potential) and increase radius
	 % in steps until corresponding potential matches surface potential
         while (surfpot-pot > eps)
         {
	   % increase radius and calculate potential at this point
           tmprad += dr;
           tmppot = Roche_potential(tmprad, pphi[p], ptheta[t]; q = mq, sph);
           % acutal potential to large -> step back and decrease stepsize to
	   % approach surface potential slower than before
           if (tmppot > surfpot)
           {
             tmprad -= dr;
             dr *= 0.1;
	     j=0;
           }
           % if the derivative negative, we are "over the hill" and outside of
	   % the Roche lobe (fill=1), and have to step back
           else if ((tmppot - pot) / dr < 0.)
           {
             if (j==0) { tmprad -= dr; dr *= 0.1; }
             else tmprad -= 10.*dr + j*dr; % go back faster if we're here again
             j++;
             pot = Roche_potential(tmprad, pphi[p], ptheta[t]; q = mq, sph);
           }
           % on the right way -> overwrite actual potential value and proceed
           else
           {
             pot = tmppot;
             j=0;
           }
         }
	 % set found radius
         pradius[p,t] = tmprad;
       }
     }
   }
 }

 % build structure
 if (qualifier_exists("chatty")) vmessage("building structure", _function_name);
 variable np = 2*ntheta*(ntheta-2)+2; % number of surface elements (rectangles)
 variable surface = struct { r = Vector_Type[np] };
 ifnot (qualifier_exists("noNorm")) surface = struct_combine(surface, struct { n = Vector_Type[np] });
 ifnot (qualifier_exists("noArea")) surface = struct_combine(surface, struct { A = Double_Type[np] });
 if (qualifier_exists("poly")) surface = struct_combine(surface, struct { p = Vector_Type[np] });
 variable rot = qualifier("rotate", 0.);
 % loop
 variable tto;
 _for t (1, ntheta-1, 1)
 {
   tto = (t-1)*(2*ntheta-2);
   _for p (0, 2*ntheta-3, 1) % exclude P = 2PI
   {
     % polygons
     if (qualifier_exists("poly"))
       surface.p[tto+p] = vector([pradius[p,t-1], pradius[p,t], t < ntheta-1 ? pradius[p+1,t] : Double_Type[0], pradius[p+1,t-1], t > 1 ? pradius[p,t-1] : Double_Type[0]],
				 [pphi[p], pphi[p], pphi[p+1], t < ntheta-1 ? pphi[p+1] : Double_Type[0], t > 1 ? pphi[p] : Double_Type[0]],
				 [ptheta[t-1], ptheta[t], t < ntheta-1 ? ptheta[t] : Double_Type[0], ptheta[t-1], t > 1 ? ptheta[t-1] : Double_Type[0]]; sph);
     % position vector
     surface.r[tto+p] = 1./(3+(t>1 && t<ntheta-1))
                        * ((t > 1 ? vector(pradius[p,t-1], pphi[p],  ptheta[t-1]; sph) : vector(0.,0.,0.)) % element including north pole is a triangle
      		           + vector(pradius[p,t], pphi[p], ptheta[t]; sph)
			   + (t < ntheta-1 ? vector(pradius[p+1,t], pphi[p+1], ptheta[t]; sph) : vector(0., 0., 0.)) % element including south pole is a triangle
			   + vector(pradius[p+1,t-1], pphi[p+1], ptheta[t-1]; sph)
			  );
     variable a, b;
     ifnot (qualifier_exists("noNorm") && qualifier_exists("noArea"))
     {
       a = vector(pradius[p,t], pphi[p], ptheta[t]; sph) - vector(pradius[p,t-1], pphi[p], ptheta[t-1]; sph);
       b = (t < ntheta-1 ? vector(pradius[p+1,t], pphi[p+1], ptheta[t]; sph) : vector(pradius[p+1,t-1], pphi[p+1], ptheta[t-1]; sph)) - vector(pradius[p,t], pphi[p], ptheta[t]; sph);
     }
     % normal vector
     ifnot (qualifier_exists("noNorm"))
     {
       surface.n[tto+p] = crossprod(a,b);
       surface.n[tto+p] = 1./vector_norm(surface.n[tto+p])*surface.n[tto+p];
     }
     % area
     ifnot (qualifier_exists("noArea"))
     {
       variable na = vector_norm(a), nb = vector_norm(b);
       surface.A[tto+p] = na*nb*sqrt(1.-(a*b)/na/nb) / ((t == 1 || t == ntheta-1) + 1.); % divide by two (area of triangle)
     }
     % evntually rotate
     if (rot > 0 && rot < 360) {
       surface.p[tto+p] = vector_rotate(surface.p[tto+p], vector(0, 0, 1), PI/180*rot);
       surface.r[tto+p] = vector_rotate(surface.r[tto+p], vector(0, 0, 1), PI/180*rot);
       surface.n[tto+p] = vector_rotate(surface.n[tto+p], vector(0, 0, 1), PI/180*rot);
     }
   }
 }

 return surface;
}
