%
% Astronomical coordinate transforms
% (no relation to the old starlink code!)
%
% J. Wilms, 2017
%

require("vector");

private define apply_transform_vector() {
    %
    % apply a coordinate transform to a vector or an array
    % of vectors
    %

    variable mat,v;
    (mat,v)=();
    
    if (Array_Type==typeof(v)) {
	variable res=Array_Type[length(v)];
	variable i;
	_for i(0,length(v)-1,1) {
	    res[i]=mat*v[i];
	}
	return res;
    }
    return mat*v;
}

private define apply_transform_coord() {
    %
    % apply the transformation matrix trans to the coordinates alpha and beta
    %

    variable trans,alpha,delta;
    (trans,alpha,delta)=();
    
    variable i;
    variable r,lambda,beta;
    if (Array_Type==typeof(alpha)) {
	variable ex=Vector_Type[length(alpha)];
	if (qualifier_exists("deg")) {
	    _for i(0,length(alpha)-1,1) {
		ex[i]=trans*vector_astro(alpha[i],delta[i];deg);
	    }
	} else {
	    _for i(0,length(alpha)-1,1) {
		ex[i]=trans*vector_astro(alpha[i],delta[i]);
	    }
	}	    
	(r,lambda,beta)=array_map(Double_Type,Double_Type,Double_Type,&vector_to_spherical,ex);
	if (qualifier_exists("deg")) {
	    lambda *=180./PI;
	    beta *= 180./PI;
	}
    } else {
	if (qualifier_exists("deg")) {
	    (r,lambda,beta)=vector_to_spherical(trans*vector_astro(alpha,delta;deg);deg);
	} else {
	    (r,lambda,beta)=vector_to_spherical(trans*vector_astro(alpha,delta));
	}
    }
    return (lambda,beta);
}

define gcrs2j2000_matrix()
%!%+
%\function{gcrs2j2000_matrix}
%\synopsis{return the frame bias matrix to convert coordinates from the GCRS to J2000.0}
%\usage{Matrix33_Type mat=gcrs2j2000_matrix();}
%\description
%  This function returns the frame bias matrix needed to convert
%  a vector in the Geocentric Celestial Reference System (GCRS)
%  into the J2000.0 dynamical reference system (see
%  astronomical almanac, section B)
%
%\seealso{precess,precession_matrix}
%!%-
{
    return matrix33_new(+0.9999999999999942, -0.000000071,        +0.00000008056,
                        +0.000000071       , +0.9999999999999969, +0.00000003306,
                        -0.00000008056     , -0.00000003306,      +0.9999999999999962);
}

define precession_matrix(JD) 
%!%+
%\function{precession_matrix}
%\synopsis{return the precession matrix for the transform from J2000.0 to a given JD}
%\usage{Matrix33_Type mat=precession_matrix(JD)}
%
%\description
% This function calculates the precession matrix for the calculation
% of the precession FROM J2000.0 TO JD using the IAU 2000.0A theory
% (Capitaine et al., 2003, A&A 412, 567, Sect. 7)
%
% To get the matrix for the conversion from JD to J2000.0, transpose the
% matrix returned by this function.
%
% In most cases users will want to use the precess-function to
% precess astronomical coordinates.
%
% Note: This is mainly an internal function, it therefore does NOT
% have the usual qualifiers such as mjd or any safety checks, however,
% for some applications it will be useful to have direct access to the
% precession matrix.
%
%\seealso{precess,precession_matrix}
%!%-
%
{
    % number of Julian centuries since J2000.0
    variable T=(JD-2451545.0)/36525.;

    % obliquity at J2000.0 (arcsec)
    variable eps0=84381.406;

    % precession angles (all in arcsec)
    variable psi=((((-0.0000000951*T+0.000132851)*T-0.00114045)*T-1.0790069)*T+5038.481507)*T;
    variable ome=((((+0.0000003337*T-0.000000467)*T-0.00772503)*T+0.0512623)*T-0.025754)*T+eps0;
    variable chi=((((-0.0000000560*T+0.000170663)*T-0.00121197)*T-2.3814292)*T+10.556403)*T;

    % convert to rad
    variable as2rad=PI/648000.;
    psi*=as2rad;
    ome*=as2rad;
    chi*=as2rad;
    eps0*=as2rad;

    % build up the precession matrix
    return matrix33_rot(3,chi)*matrix33_rot(1,-ome)*matrix33_rot(3,-psi)*matrix33_rot(1,eps0);
}

define precession_matrix2()
%!%+
%\function{precession_matrix2}
%\synopsis{return the precession matrix for the transform between two equinoxes}
%\usage{Matrix33_Type mat=precession_matrix2(fromJD,toJD)}
%
%\description
% This function calculates the precession matrix for the calculation
% of the precession from fromJD TO toJD using the IAU 2000.0A theory
% (Capitaine et al., 2003, A&A 412, 567, Sect. 7)
%
% In most cases users will want to use the precess-function to
% precess astronomical coordinates.
%
% Note: This is mainly an internal function, it therefore does NOT
% have the usual qualifiers such as mjd or any safety checks, however,
% for some applications it will be useful to have direct access to the
% precession matrix.
%
%\seealso{precess,precession_matrix}
%!%-
%
{
    variable fromequinox,toequinox;
    (fromequinox,toequinox)=();

    %
    % build the precession matrix: precess from fromequinox to J2000 and
    % then from J2000 to toequinox.
    % J2000 = JD2451545
    %
    variable mtoj2000,mfromj2000;

    if (fromequinox==toequinox) {
	return matrix33_identity();
    }
    
    if (fromequinox != 2451545.0) {
	% from different from J2000.0?
	mtoj2000=(precession_matrix(fromequinox)).transpose(); % transform epoch->J2000
	if (toequinox == 2451545.0) {
	    return mtoj2000;
	}
    } 

    if (toequinox != 2451545.0) {
	% to different from J2000.0?
	mfromj2000=precession_matrix(toequinox); % transform J2000->epoch
	if (fromequinox == 2451545.0) {
	    return mfromj2000;
	}
    }
    % general case: product of both transforms
    return mfromj2000*mtoj2000;
}

define precess()
%!%+
%\function{precess}
%\synopsis{precess equatorial coordinates between two equinoxes}
%\usage{(alpha,delta)=precess(alpha,delta;qualifiers)}
%\altusage{AstroVector3_Type ecl=precess(eqp;qualifiers)}
%\qualifiers{
%\qualifier{fromequinox}{starting equinox of the transformation.
%                      Float: JD, string: epoch designation (e.g.,
%                      "J2000.0" or "B1950.0";
%                      default: J2000.0)}
%\qualifier{toequinox}{end equinox of the transformation (same interpretation)
%                    as fromequinox; required!}
%\qualifier{deg}{interpret angular arguments in degrees (default is radians!)
%                applies also to the return value.}
%\qualifier{mjd}{interpret equinoxes as a MJD (default: JD)}
%}
%\description
%
% Precess a coordinate from one equinox to another using
% the IAU 2000A model.
%
% The default equinox is J2000.0, formally the equinoxes are in TDB.
%
%\seealso{Vector_Type,JDofEpoch,dms2deg,hms2deg,precession_matrix,gcrs2j2000_matrix}
%!%-
{
    variable alpha,delta;
    switch(_NARGS)
    { case 1: alpha=();}
    { case 2: (alpha,delta)=();}
    { help(_function_name()); return; }

    %
    % set the epochs
    %
    variable fromequinox,toequinox;
    fromequinox=JDofEpoch(qualifier("fromequinox","J2000.0");;__qualifiers());
    if (qualifier_exists("toequinox")) {
	toequinox=JDofEpoch(qualifier("toequinox");;__qualifiers()); % take into account mjd qualifier
    } else {
	throw UsageError, sprintf("%s: toequinox qualifier is mandatory\n",_function_name());
    }

    % final transformation matrix
    variable transform=precession_matrix2(fromequinox,toequinox);

    %
    % Apply the coordinate transform
    %
    if (_NARGS==1) {
	%  argument is a vector: transform the vector
	return apply_transform_vector(transform,alpha);
    } 

    % arguments are coordinates
    if (qualifier_exists("deg")) {
	return apply_transform_coord(transform,alpha,delta;deg);
    }
    return apply_transform_coord(transform,alpha,delta);    
}

define nutation_matrix()
%!%+
%\function{nutation_matrix}
%\synopsis{Calculate the nutation matrix for the mean equinox JD}
%\usage{Matrix33_Type N=nutation_matrix(JD)}
%\qualifiers{
%   \qualifier{theory}{nutation theory to use, see help for
%                      function nutation_angles for details.
%                      Default: NU2000K}
%}
%\description
% This function returns the nutation matrix to convert coordinates
% for the mean equinox of JD to the true equinox of JD.
% 
% Note: This is mainly an internal function, it therefore does NOT
% have the usual qualifiers such as mjd, however, for some applications
% it is useful to have direct access to the nutation matrix
%!%-
{
    variable JD=();
    variable theory=qualifier("theory","NU2000K");

    variable dpsi,deps;
    (dpsi,deps)=nutation_angles(JD;theory=theory);

    % obliquity
    variable eps=obliquity(JD); % rad!
    % true obliquity
    variable epstrue=eps+deps;

    return matrix33_rot(1,-epstrue)*matrix33_rot(3,-dpsi)*matrix33_rot(1,eps);
}

define equatorial2ecliptical()
%!%+
%\function{equatorial2ecliptical}
%\synopsis{convert equatorial (alpha,delta) to ecliptical (lambda,beta) coordinates}
%\usage{(lambda,beta)=equatorial2ecliptical(alpha,delta;qualifiers)}
%\altusage{Vector_Type ecl=equatorial2ecliptical(eqp;qualifiers)}
%\qualifiers{
%\qualifier{equinox}{equinox of the transformation. Float: JD, string: equinox
%                  designation (e.g., "J2000.0" or "B1950.0";
%                  default: J2000.0)}
%\qualifier{deg}{interpret angular arguments in degrees (default is radians!)
%                applies also to the return value.}
%\qualifier{mjd}{interpret equinox as a MJD (default: JD)}
%\qualifier{inverse}{perform the conversion from ecliptical to equatorial
%                    coordinates (see ecliptical2equatorial() for an
%                    equivalent interface)}
%\qualifier{true}{perform calculation for apparent coordinates, i.e., include nutation terms}
%}
%\description
%  The routine takes coordinates in the equatorial (right ascension, 
%  declination) system and converts them into ecliptical coordinates 
%  using the obliquity of the ecliptic for the equinox and vice versa..
%
%  Alternatively, the routine also accepts an Vector_Type and then
%  returns a vector in ecliptical coordinates (or vice versa if the inverse
%  qualifier is given).
%
%  The default equinox is J2000.0.
%
%\seealso{Vector_Type, ecliptical2equatorial, JDofEpoch,dms2deg,hms2deg}
%!%-
{
    variable alpha,delta;
    switch(_NARGS)
    { case 1: alpha=();}
    { case 2: (alpha,delta)=();}
    { help(_function_name()); return; }

    %
    % get the obliquity
    % because __qualifiers() would also transfer the deg keyword,
    % do NOT just transfer everything 
    %
    variable epsilon;
    if (qualifier_exists("true")) {
	if (qualifier_exists("equinox")) {
	    if (qualifier_exists("mjd")) {
		epsilon=obliquity(JDofEpoch(qualifier("equinox"));mjd,true);
	    } else {
		epsilon=obliquity(JDofEpoch(qualifier("equinox"));true);
	    }
	} else {
	    epsilon=obliquity(JDofEpoch("J2000.0");true);
	}
    } else {
	if (qualifier_exists("equinox")) {
	    if (qualifier_exists("mjd")) {
		epsilon=obliquity(JDofEpoch(qualifier("equinox"));mjd);
	    } else {
		epsilon=obliquity(JDofEpoch(qualifier("equinox")));
	    }
	} else {
	    epsilon=obliquity(JDofEpoch("J2000.0"));
	}
    }
    %
    % rotation matrix
    %
    variable transform;
    if (qualifier_exists("inverse")) {
	transform=matrix33_rot(1,-epsilon);
    } else {
	transform=matrix33_rot(1,epsilon);
    }

    %
    % Apply the coordinate transform
    %
    if (_NARGS==1) {
	%  argument is a vector: transform the vector
	return apply_transform_vector(transform,alpha);
    } 

    % arguments are coordinates
    if (qualifier_exists("deg")) {
	return apply_transform_coord(transform,alpha,delta;deg);
    }
    return apply_transform_coord(transform,alpha,delta);    
}

define ecliptical2equatorial()
%!%+
%\function{ecliptical2equatorial}
%\synopsis{convert ecliptical (lambda,beta) to equatorial coordinates}
%\usage{(alpha,delta)=ecliptical2equatorial(lambda,beta;qualifiers)}
%\altusage{Vector_Type eqp=equatorial2ecliptical(ecl;qualifiers)}
%\qualifiers{
%\qualifier{equinox}{equinox of the transformation. Float: JD, string: equinox
%                  designation (e.g., "J2000.0" or "B1950.0";
%                  default: J2000.0)}
%\qualifier{deg}{interpret angular arguments in degrees (default is radians!)
%                applies also to the return value.}
%\qualifier{mjd}{interpret equinox as a MJD (default: JD)}
%\qualifier{true}{perform calculation for apparent coordinates, i.e., include nutation terms}
%}
%\description
%  The routine takes coordinates in the ecliptical (lambda, beta)
%  system and converts them into equatorial coordinates (right ascension,
%  declination), using the obliquity of the ecliptic for the equinox
%  and vice versa.
%
%  Alternatively, the routine also accepts a Vector_Type and then
%  returns an Vector_Type vector in equatorial coordinates (or
%  vice versa if the inverse qualifier is given).
%
%  The default equinox is J2000.0.
%
%  This routine just calls equatorial2ecliptical with all arguments and the
%  inverse qualifier, but is a simpler interface.
%
%\seealso{Vector_Type, equatorial2ecliptical, JDofEpoch,dms2deg,hms2deg}
%!%-
{
    if (_NARGS<1 or _NARGS>2) {
	help(_function_name()); return;
    }

    % sanity check, otherwise havoc ensues!
    if (qualifier_exists("inverse")) {
	throw UsageError, sprintf("%s: inverse qualifier is NOT allowed!\n",_function_name());
    }
    variable qual=__qualifiers();
    qual=struct { @qual, "inverse" };

    % I am sure there is a more elegant way to do this...
    variable alpha,delta;
    if (_NARGS==1) {
	alpha=();
	return equatorial2ecliptical(alpha ;;qual);
    }
    (alpha,delta)=();
    return equatorial2ecliptical(alpha,delta ;;qual);
}

define equatorial2horizon()
%!%+
%\function{equatorial2horizon}
%\synopsis{convert equatorial (alpha,delta) to horizon (elevation, azimuth) coordinates}
%\usage{(azi,ele)=equatorial2horizon(alpha,delta;qualifiers)}
%\altusage{Vector_Type hor=equatorial2horizon(eqp;qualifiers)}
%\qualifiers{
% \qualifier{JD}{JD for which the calculation is to be performed (in UT1/UTC). Mandatory.}
% \qualifier{lon}{geographic longitude of the observer, positive towards the east. Mandatory.}
% \qualifier{lat}{geographic latitude of the observer, positive towards the north. Mandatory.}
% \qualifier{deg}{interpret all angular arguments in degrees (default is radians!)
%                 applies also to the return value.}
% \qualifier{mjd}{interpret JD argument as a MJD (default: JD)}
% \qualifier{inverse}{perform the conversion from horizontal to equatorial
%                     coordinates (see horizontal2equatorial() for an
%                     equivalent interface)}
%}
%\description
%  The routine takes coordinates in the equatorial (right ascension, 
%  declination) system for the equinox and ecliptic of the date
%  (i.e., "apparent coordinates"), and converts them into the elevation
%  and azimuth for a given observer's position and time. Note that
%  the JD, lon, and lat qualifiers are mandatory!
%
%  The JD argument is to be measured in UT1, for most cases it is sufficient to
%  approximate this as UT. Note that ephemerides work in TDB or TT, which has
%  an offset of several tens of seconds. For most applications this should not
%  matter, especially because of the uncertainty of refraction, but please take
%  this into account if highest precision is needed.
%
%  Formally the coordinates have to be topocentric coordinates in
%  order to include parallax effects. For most applications this is not 
%  important as long as a precision of a few arcsec or worse is needed.
%  Eventually a routine applying all of these effects will be provided.
%
%  Alternatively, the routine also accepts an Vector_Type and then
%  returns a vector in horizon coordinates (or vice versa if the inverse
%  qualifier is given).
%
%\seealso{Vector_Type, horizontal2equatorial, JDofEpoch,dms2deg,hms2deg,tai2tt,utc2tai,tt2tdb}
%!%-
{
    variable alpha,delta,JD;
    switch(_NARGS)
    { case 1: (alpha)=();}
    { case 2: (alpha,delta)=();}
    { help(_function_name()); return; }

    variable lon,lat;

    if (qualifier_exists("lon")) {
	lon=qualifier("lon");
    } else {
	throw UsageError, sprintf("%s: lon qualifier is mandatory\n",_function_name());
    }

    if (qualifier_exists("lat")) {
	lat=qualifier("lat");
    } else {
	throw UsageError, sprintf("%s: lat qualifier is mandatory\n",_function_name());
    }

    if (qualifier_exists("deg")) {
	lon*=PI/180.;
	lat*=PI/180.;
    }
    
    if (qualifier_exists("JD")) {
	JD=qualifier("JD");
    } else {
	throw UsageError, sprintf("%s: JD qualifier is mandatory\n",_function_name());
    }
    if (qualifier_exists("mjd")) {
	JD=mjd2jd(JD);
    }
    
    %
    % local siderial time
    %
    variable lst=GMST(JD;lst,lon=lon,lat=lat);
    
    % s->rad
    lst=2.*PI*lst/86400.;

    %
    % rotation matrix
    %
    variable transform;
    if (qualifier_exists("inverse")) {
	% transform from H to HA system
	variable transform_h2ha=matrix33_rot(2,lat-PI/2.)*matrix33_rot(3,PI);
	% transform from HA to RA system
	variable transform_ha2ra=matrix33_rot(3,-lst)*matrix33_reflect(2);
	transform=transform_ha2ra*transform_h2ha;
    } else {
	% transform from RA to HA system
	variable transform_ra2ha=matrix33_reflect(2)*matrix33_rot(3,lst);
	% transform from HA to H system
	variable transform_ha2h=matrix33_rot(3,PI)*matrix33_rot(2,PI/2.-lat);
	transform=transform_ha2h*transform_ra2ha;

	variable t1=matrix33_new(cos(lst),sin(lst),0.,
 	                         sin(lst),-cos(lst), 0.,
	                         0.,0.,1.);
	variable t2=matrix33_new(sin(lat),0,-cos(lat),
	                         0,1,0,
	                         cos(lat),0,sin(lat));
	transform=t2*t1;
%	variable tt=t2*t1;
%	print(tt.as_array());
%	print("XXXX");
%	print(transform.as_array());
%	exit;

	
    }

    %
    % Apply the coordinate transform
    %
    if (_NARGS==1) {
	%  argument is a vector: transform the vector
	return apply_transform_vector(transform,alpha);
    }

    % arguments are coordinates
    if (qualifier_exists("deg")) {
	return apply_transform_coord(transform,alpha,delta;deg);
    }
    return apply_transform_coord(transform,alpha,delta);    
}

define horizon2equatorial()
%!%+
%\function{horizon2equatorial}
%\synopsis{convert horizon (azi,ele) coordinates to equatorial (alpha,delta)}
%\usage{(alpha,delta)=horizon2equatorial(azi,ele;qualifiers)}
%\altusage{Vector_Type eqp=horizon2equatorial(hor;qualifiers)}
%\qualifiers{
% \qualifier{JD}{JD for which the calculation is to be performed. Mandatory.}
% \qualifier{lon}{geographic longitude of the observer, positive towards the east. Mandatory.}
% \qualifier{lat}{geographic latitude of the observer, positive towards the north. Mandatory.}
% \qualifier{deg}{interpret all angular arguments in degrees (default is radians!)
%                 applies also to the return value.}
% \qualifier{mjd}{interpret date as a MJD (default: JD)}
%}
%
%\description
% The function  takes coordinates in the horizon (azimuth,elevation) system
% and converts them into the equatorial system for the ecliptic and
% elevation of the date. Alternatively the routine also accepts a
% Vector_Type and then returns avector in equatorial coordinates.
%
% See the description of the function equatorial2horizon for
% references and further information. This routine is just a
% wrapper around that function, with the qualifier "inverse" set.
%
%\seealso{Vector_Type, equatorial2horizon,dms2deg,hms2deg}
%!%-
{
    if (_NARGS<1 or _NARGS>2) {
	help(_function_name()); return;
    }

    % sanity check, otherwise havoc ensues!
    if (qualifier_exists("inverse")) {
	throw UsageError, sprintf("%s: inverse qualifier is NOT allowed!\n",_function_name());
    }
    variable qual=__qualifiers();
    qual=struct { @qual, "inverse" };

    % I am sure there is a more elegant way to do this...
    variable azi,ele;
    if (_NARGS==1) {
	azi=();
	return equatorial2horizon(azi ;;qual);
    }
    (azi,ele)=();
    return equatorial2horizon(azi,ele ;;qual);
}

define equatorial2galactic()
%!%+
%\function{equatorial2galactic}
%\synopsis{convert J2000.0 equatorial (alpha,delta) to galactic (l,b) coordinates}
%\usage{(l,b)=equatorial2galactic(alpha,delta;qualifiers)}
%\altusage{Vector_Type gal=equatorial2galactic(eqp;qualifiers)}
%\qualifiers{
%\qualifier{deg}{interpret angular arguments in degrees (default is radians!)
%                applies also to the return value.}
%\qualifier{inverse}{perform the conversion from galactic to equatorial
%                    coordinates (see galactic2equatorial() for an
%                    equivalent interface)}
%\qualifier{blaauw}{use the original Blaauw definition of the IAU galactic
%              coordinate system; see below for caveats}
%}
%\description
%  The function takes coordinates in the equatorial (right ascension, 
%  declination) system and converts them into galactic
%  coordinates (or vice versa, if the inverse qualifier is given) for
%  the IAU 1958 Galactic coordinate system (Blaauw et al., 1960,
%  MNRAS 121, 123).
%
%  Alternatively, the routine also accepts a Vector_Type and then
%  returns a vector in galactic coordinates (or vice versa if the 
%  inverse qualifier is given).
%
%  The function is for J2000.0 coordinates only, use the
%  precess function to convert to that equinox if needed.
%
%  See Lane (1979, PASP 91, 405) for a discussion of the subtleties of
%  this conversion and Johnson & Soderblom (1987, AJ 93, 864) for a
%  non-standard derivation of the conversion matrix (the one used here
%  is basically identical, but uses proper Euler angles). By default, 
%  the conversion is done using the precessed pole and inclination for 
%  the FK5 system (J2000.0) given by Liu et al (2011, A&A 526, A16), 
%  which is a thorough derivation of values that are essentially 
%  identical to those also given in the appendix of Reid & Brunthaler
%  (2004, ApJ 616, 872).
%
%  If the qualifier Blaauw is given, then the coordinates are first 
%  precessed to B1950.0 and then transformed. Note that this approach
%  is not good for the highest precision, since no conversion from,
%  e.g., FK5 to FK4 coordinates is performed as this would result in a
%  non-orthogonal coordinate system. Please use this qualifier only
%  if you know what you are doing and read Liu et al. (2011) before
%  using the qualifier.
%
%\seealso{Vector_Type, galactic2equatorial,precess,dms2deg,hms2deg}
%!%-
{
    variable alpha,delta;
    switch(_NARGS)
    { case 1: alpha=();}
    { case 2: (alpha,delta)=();}
    { help(_function_name()); return; }

    variable alpha_ngp,delta_ngp,theta0;
    if (qualifier_exists("blaauw")) {
	% build conversion matrix to galactic coordinates
	% (positions in B1950 by Blaauw)
	alpha_ngp=192.25*PI/180.;
	delta_ngp=27.4*PI/180.;
	theta0=123.*PI/180.;
    } else {
	% values for J2000.0 per Liu et al. (2011, eq.10)
	alpha_ngp=hms2deg(12,51,26.27549;radian);
	delta_ngp=dms2deg(27,07,41.7043;radian);
	theta0=122.93191857*PI/180.;
    }

    variable transform;

    if (qualifier_exists("inverse")) {
	transform= matrix33_rot(3,3.*PI/2.-alpha_ngp)*matrix33_rot(1,-(PI/2.-delta_ngp))*matrix33_rot(3,theta0-PI/2.);
    } else {
	%
	% transformation equatorial->galactic using Euler angles
	% (reproduces Johnson & Soderblom, section III, or
	% Liu 2011, but uses Euler angles and not the
	% non-rotation matrices that are used by Johnson & Soderblom )
	transform=matrix33_rot(3,PI/2.-theta0)*matrix33_rot(1,PI/2.-delta_ngp)*matrix33_rot(3,-(3.*PI/2.-alpha_ngp));

	% Johnson & Soderblom, section III (for B1950.0)
	%variable trans=matrix33_new(-0.06699,-0.87276,-0.48354,
	%                            +0.49273,-0.45035,+0.74458,
	%                            -0.86760,-0.18837,+0.46020);
    }

    if (qualifier_exists("blaauw")) {
	%
	% go to B1950.0 and use original Blaauw coordinates
	%
	
	% precess input coordinates to B1950.0
	variable prectrans=precession_matrix(JDofEpoch("B1950.0"));
	if (qualifier_exists("inverse")) {
	    transform=prectrans*transform;
	} else {
	    transform=transform*prectrans;
	}
    }
    
    %
    % Apply the coordinate transform
    %
    if (_NARGS==1) {
	%  argument is a vector: transform the vector
	return apply_transform_vector(transform,alpha);
    } 

    % arguments are coordinates
    if (qualifier_exists("deg")) {
	return apply_transform_coord(transform,alpha,delta;deg);
    }
    return apply_transform_coord(transform,alpha,delta);    
}

define galactic2equatorial()
%!%+
%\function{galactic2equatorial}
%\synopsis{convert galactic (l,b) coordinates to J2000.0 equatorial (alpha,delta)}
%\usage{(alpha,delta)=galactic2equatorial(l,b;qualifiers)}
%\altusage{Vector_Type eqp=equatorial2galactic(gal;qualifiers)}
%\qualifiers{
%\qualifier{deg}{interpret angular arguments in degrees (default is radians!)
%                applies also to the return value.}
%}
%
%\description
% The function  takes coordinates in the galactic (l,b) system
% and converts them into the equatorial J2000.0 system. Alternatively
% the routine also accepts a Vector_Type and then returns a
% vector in equatorial coordinates for J2000.0.
%
% The function is for J2000.0 coordinates only, use the
% precess function to convert the result to another equinox
% if needed.
%
% See the description of the function equatorial2galactic for
% references and further information. This routine is just a
% wrapper around that function, with the qualifier "inverse" set.
%
%\seealso{Vector_Type, equatorial2galactic,precess,dms2deg,hms2deg}
%!%-
{
    if (_NARGS<1 or _NARGS>2) {
	help(_function_name()); return;
    }

    % sanity check, otherwise havoc ensues!
    if (qualifier_exists("inverse")) {
	throw UsageError, sprintf("%s: inverse qualifier is NOT allowed!\n",_function_name());
    }
    variable qual=__qualifiers();
    qual=struct { @qual, "inverse" };

    % I am sure there is a more elegant way to do this...
    variable ell,b;
    if (_NARGS==1) {
	ell=();
	return equatorial2galactic(ell ;;qual);
    }
    (ell,b)=();
    return equatorial2galactic(ell,b ;;qual);
}

