define g_earth()
%!%+
%\function{g_earth}
%\synopsis{Calculates the acceleration at the Earth's surface as a function of latitude}
%\usage{Double_Type g = g_earth(Double_Type phi);}
%
%\qualifiers{
%\qualifier{deg}{phi is in degrees, not in radian.}
%\qualifier{altitude}{altitude of the observer above the geoid [m]}
%\qualifier{mks}{return g in m/s^2, not in cm/s^2}
%\qualifier{allen}{use the equation of Allen (1973, Astrophys. Quantities)}
%\qualifier{almanac}{Urban & Seidelman (2013; Expl. Suppl. Astron. Almanac)}
%}
%\description
%    This code calculates the surface acceleration on the earth as
%    a function of the latitude (in rad). The equations used are 
%    from a compilation of different approximations to g by
%    Mangum & Wallace (2015), PASP 127, 74.
%
%    The default returned is for the WGS84 geoid.
%
%    This routine is array safe in phi or altitude.
%\seealso{dms2deg}
%!%-
{
    variable phi;
    switch(_NARGS)
    { case 1: phi = (); }
    { help(_function_name()); return; }

    if (qualifier_exists("deg")) {
	phi*=PI/180.;
    }


    variable h0=qualifier("altitude",0.)/1000.; % m->km

    variable g,sin2phi,sinphi,cosphi,cos2phi;
    if (qualifier_exists("allen")) {
	sinphi=sin(phi);
	sin2phi=sin(2.*phi);
	g=9.757823*(1.+0.005301*sinphi*sinphi-0.0000059*sin2phi*sin2phi-0.000284*h0);
    } else {
	if (qualifier_exists("almanac")) {
	    cos2phi=cos(2.*phi);
	    g=9.784*(1.-0.0026*cos2phi-0.00028*h0);
	} else {
	    % WGS84
	    sinphi=sin(phi);
	    g=9.7803267714*((1.+0.00193185138639*sinphi*sinphi)/sqrt(1.-0.00669437999013*sinphi*sinphi))-0.02253-0.0027774*h0;

	}
    }
    if (qualifier_exists("mks")){
	return g;
    }
    return g*100.;
}
