%%%%%%%%%%%%%%%%%
private define ldist(z)
%%%%%%%%%%%%%%%%%
{
  variable omega_lambda = qualifier("omega_lambda",   0.7);
  variable           q0 = qualifier("q0"          , -0.55);

  variable term1 = (1.+z)^2;
  variable term2 =  1.+2.*(q0+omega_lambda)*z;
  variable term3 = z*(2.+z)*omega_lambda;
  variable denom = (term1*term2 - term3);

  return 1./sqrt(denom);
}

%%%%%%%%%%%%%%%%%
define lumdist()
%%%%%%%%%%%%%%%%%
%!%+
%\function{lumdist}
%\synopsis{Calculate luminosity distance (in Mpc) of an object given its redshift}
%\usage{result = lumdist(z);}
%\qualifiers{
%\qualifier{silent}{If set, the program will not display adopted
%                       cosmological parameters at the terminal.}
%\qualifier{h0}{[=70] Hubble parameter in km/s/Mpc}
%\qualifier{omega_k}{curvature constant, normalized to the closure density.
%                       Default is 0, indicating a flat universe.}
%\qualifier{omega_m}{Matter density, normalized to the closure density,
%                       default is 0.3. Must be non-negative.}
%\qualifier{omega_lambda}{: Cosmological constant, normalized to the
%                       closure density (default: 0.7).}
%\qualifier{q0}{[=-0.55] Deceleration parameter, numeric scalar = -R*(R'')/(R')^2}
%}
%\description
%    INPUTS:    z = redshift (positive scalar or vector)
%    OUTPUTS:   The result of the function is the luminosity distance (in Mpc)
%               for each input value of z.
%    The luminosity distance in the Friedmann-Robertson-Walker model is 
%    taken from  Caroll, Press, and Turner (1992, ARAA, 30, 499), p. 511.
%    Uses a closed form (Mattig equation) to compute the distance when the 
%    cosmological constant is zero.   Otherwise integrates the function using
%    QSIMP.	
%    See help for cosmo_param for a description of the cosmological parameters,
%    note that only two out of the four parameters should be given!
%
%    The routine can fail to converge at high redshift for closed universes with
%    non-zero lambda. This can presumably be fixed by replacing QSIMP with
%    an integrator that can handle a singularity.
%
%\example
%    %  Plot the distance of a galaxy in Mpc as a function of redshift out 
%    %  to z = 5.0, assuming the default cosmology (Omega_m=0.3, Lambda = 0.7,
%    %  H0 = 70 km/s/Mpc)
%    variable z = [0:5:0.1];
%    plot (z, lumdist(z));
%
%    %  Now overplot the relation for zero cosmological constant and 
%    %  Omega_m=0.3
%    oplot (z, lumdist(z ; omega_lambda=0 , omega_m=0.3) );
%\seealso{cosmo_param, qsimp}
%!%-
{
  variable           h0 = qualifier("h0"     ,         70);
  variable      omega_k = qualifier("omega_k",       NULL); 
  variable      omega_m = qualifier("omega_m",       NULL);
  variable omega_lambda = qualifier("omega_lambda",  NULL);
  variable           q0 = qualifier("q0"     ,       NULL);
  variable            c = qualifier("c"      ,   2.9979e5); % speed of light in km/s 

  variable z;
  switch(_NARGS)
  {
   case 1: z = ();
  }
  { help(_function_name()); return; }

  variable n = length(z);

  % obtain cosmological params from cosmo_param
  variable cosmopar = cosmo_param(; omega_k = omega_k, omega_lambda = omega_lambda,
				  omega_m = omega_m, q0 = q0);
  omega_k      = cosmopar.omega_k;
  omega_lambda = cosmopar.omega_lambda;
  omega_m      = cosmopar.omega_m;
  q0           = cosmopar.q0;
  
  ifnot (qualifier_exists ("silent"))
    vmessage("lumdist:\nh0: %5.2f, omega_m: %5.2f, omega_lambda: %5.2f, q0: %5.2f, omega_k: %5.2f"
	     , h0, omega_m, omega_lambda, q0, omega_k );

  % For the case of lambda = 0, we use the closed form from equation 5.238 of
  % Astrophysical Formulae (Lang 1998). This avoids terms that almost cancel
  % at small q0*z better than the more familiar Mattig formula.
  variable dlum;
    
  if (omega_lambda == 0)
  {
    variable denom = sqrt(1.+2.*q0*z) + 1. + q0*z;
    dlum = (c*z/h0)*(1. + z*(1.-q0)/denom);
    return dlum;
  } else {
    % For non-zero lambda 
    variable i;
    variable tz = typeof(z);
    if (tz != Array_Type) z = [z];
    dlum = z*0.;
    _for i(0,n-1,1)
    {
      if (z[i] <= 0.0) dlum[i] = 0.0;
      else
      {
	dlum[i] = qsimp ( &ldist, 0, z[i] ;; struct_combine ( __qualifiers, struct {q0 = q0, omega_lambda = omega_lambda}) );
      }
    }
    
    if (omega_k > 0.) dlum = sinh(sqrt(omega_k)*dlum)/sqrt(omega_k);
    if (omega_k < 0.) 
    {
      dlum = sin(sqrt(-omega_k)*dlum)/sqrt(-omega_k);
      if (dlum < 0) dlum = 0;
    }

    if (tz != Array_Type) { z = z[0]; dlum = dlum[0]; }
    return, c*(1.+z)*dlum/h0;
  }
}
