define object_visibility_satellite(ra,dec,mjdstart,mjdstop) {
%!%+
%\function{object_visibility_satellite}
%\synopsis{Calculate the time intervals when an object is observable with an astronomical satellite}
%\usage{visibility=object_visibility_satellite(ra,dec,mjdstart,mjdstop);}
%\qualifiers{
%\qualifier{dt}{Time resolution of the visibility calculation (days, default: 1)}
%\qualifier{numerical}{see below} 
%
%    Qualifiers to select the satellite. Constraints are ANDed if multiple
%    satellite qualifiers are given, i.e., the joint visibility is returned
%\qualifier{swift}{46-180 deg from Sun; so far ignores lunar constraint (>23deg)}
%\qualifier{suzaku}{70-110 deg from Sun}
%\qualifier{xmm}{70-110 deg from Sun}
%\qualifier{chandra}{>46deg from Sun; ignores lunar constraint (>6deg)}
%\qualifier{hst}{>50deg from Sun}
%\qualifier{nustar}{no solar avoidance constraints(!)}
%}
%\description
% This routine calculates the time intervals during which an astronomical
% source with position ra, dec (both given in degrees) is visible to a given
% astronomical satellite. Most popular facilities are supported (see above).
% For each time between mjdstart and mjdstop the routine calculates the
% angular separation between the source and the Sun. By default it returns
% an array of structures giving the time intervals during which a source is
% visible. The tags of the structure describing each interval are 
%    MJDstart and MJDstop: start and stop MJD of the visibility interval
%    MJDstring           : human readable string of the above
% An empty array is returned if the object is not visible during the time
% interval specified by mjdstart and mjdstop.
%
% If the qualifier "numerical" is set, the function returns a structure with
% the following tags
%    mjd: MJD
%    theta: separation betwee source and Sun (deg)
%    visible: boolean visibility (0: no, 1: visible)
% instead of the array of time intervals
%
% Note: if the visibility interval starts before mjdstart or extends beyond
%   mjdend, the start and stop times of the search interval are shown instead.
% Note 2: several satellites also have moon avoidance angles, or avoidance
%   angles for the Earth. Both are NOT (yet) taken into account.
%
%\example
% % convert source position into degrees
% variable ra=hms2deg(19,49,35.49);
% variable dec=dms2deg(+30,12,31.8);
%
% % visibility during the year 2013
% variable tstart=MJDofDate(2013,1,1);
% variable tstop=tstart+365;
%
% variable vis=object_visibility_satellite(ra,dec,tstart,tstop;suzaku,dt=0.1);
%
% variable i;
% for(i=0;i<length(vis);i++) {
%     printf("%s\n",vis[i].MJDstring);
% }
%!%-
    variable dt=qualifier("dt",1.);
    variable tstart=mjdstart;

    variable thmin=0.;
    variable thmax=360.;

    if (qualifier_exists("swift")) {
	thmin=max([thmin,46.]); 
	thmax=min([thmax,180.]);
	% also: moon: 23deg (???)
    }

    if (qualifier_exists("suzaku")) {
	thmin=max([thmin,70.]); 
	thmax=min([thmax,110.]);
    }

    if (qualifier_exists("xmm") || qualifier_exists("xmm-newton")) {
	thmin=max([thmin,70.]); 
	thmax=min([thmax,110.]);
    }

    if (qualifier_exists("chandra")) {
	thmin=max([thmin,46.]); 
	thmax=min([thmax,180.]);
	% also: moon: outside 6deg
    }

    if (qualifier_exists("hst")) {
	thmin=max([thmin,50.]); 
	thmax=min([thmax,180.]);
    }

    if (qualifier_exists("nustar")) {
	thmin=max([thmin,0.]); 
	thmax=min([thmax,180.]);
    }

    variable visibility=struct {mjd=[mjdstart:mjdstop:dt]};
    variable ilen=length(visibility.mjd);

    visibility=struct_combine(visibility,
                           struct { theta=Double_Type[ilen],
                                    visible=Int_Type[ilen]
                                  }
    );

    variable i;
    _for i (0, ilen-1, 1) {
	variable eqsun,eclsun;
	(eqsun,eclsun)=sunpos(visibility.mjd[i];MJD);

	variable dist=greatcircle_distance(ra,dec,eqsun.ra,eqsun.dec;unit="deg");
	dist*=180./PI;

	visibility.theta[i]=dist;
	visibility.visible[i]=0; % false
	if (dist>=thmin && dist <=thmax) {
	    visibility.visible[i]=1; % true
	}
    }

    if (qualifier_exists("numerical")) {
	return visibility;
    }

    %
    % convert visibility list into something easy to understand
    %
    variable vislist={};

    variable vis=0;
    variable visint=struct{MJDstart,MJDstop,MJDstring};
    _for i(0,ilen-1,1) {
	% new visibility interval?
	if (vis==0 && visibility.visible[i]==1) {
	    visint.MJDstart=visibility.mjd[i];
	    vis=1;
	}
	% end of current visibility interval
	% NB: i is guaranteed to be >1
	if (vis==1 && visibility.visible[i]==0) {
	    visint.MJDstop=visibility.mjd[i-1];
	    visint.MJDstring=strftime_MJD(visint.MJDstart)+" -- "+
	    strftime_MJD(visint.MJDstop);
	    list_append(vislist,@visint);
	    vis=0;
	}
    }
    % end of loop, is the interval still open?
    if (vis==1) {
	visint.MJDstop=visibility.mjd[ilen-1];
	visint.MJDstring=strftime_MJD(visint.MJDstart)+" -- "+
 	                 strftime_MJD(visint.MJDstop);
	list_append(vislist,visint);
    }
    if (length(vislist)==0) {
	visibility=[0:-1];
    } else {
	visibility=list_to_array(vislist);
    }

    return(visibility);
}
