%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
define radial_velocity_binary()
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%!%+
%\function{radial_velocity_binary}
%\synopsis{computes the radial velocity of a binary system as a function of orbital phase or time}
%\usage{Double_Type v = radial_velocity_binary(Double_Type x);}
%\qualifiers{
%\qualifier{v0}{systemic velocity}
%\qualifier{K}{velocity semi amplitude, in km/s}
%\qualifier{asini}{value of a * sin i, in kilometers}
%\qualifier{P}{orbital period, in days}
%\qualifier{T0}{epoch of periastron, x means time in days}
%\qualifier{T90}{epoch of mean longitude 90 degr, x means time in days}
%\qualifier{e}{eccentricity (0 <= e < 1)}
%\qualifier{omega}{longitude of periastron in radian (unless degrees is set), default = 0}
%\qualifier{degrees}{omega is measured in degrees instead of radian}
%}
%\description
%    x has the meaning of orbital phase (0 <= x < 1) unless T0 or T90 is specified.
%\seealso{Ch. 3 of R.W. Hilditch, An Introduction to Close Binary Stars, Cambridge Univ. Press, 2001}
%!%-
{
  variable x;
  switch(_NARGS)
  { case 1: x = (); }
  { help(_function_name()); return; }

  variable e = qualifier("e", 0);
  variable K = qualifier("K", 1);
  variable asini = qualifier("asini", 0);
  variable P = qualifier("P", 0);
  variable omega = qualifier("omega", 0);
  if(qualifier_exists("degrees"))  omega *= PI/180.;
  variable v0 = qualifier("v0", 0);

  if(e<0. || e>=1.)
  { vmessage("error (%s): Eccentricity must be between 0 and 1.", _function_name());
    return;
  }
  if(qualifier_exists("T0") && qualifier_exists("T90"))
  { vmessage("error (%s): Only one qualifier of 'T0' and 'T90' is allowed.", _function_name());
    return;
  }
  if(qualifier_exists("K"))
  { if(asini!=0)
    { vmessage("error (%s): Either  K  *or*  asini and P  have to be specified.", _function_name());
      return;
    }
  }
  else
    if(asini!=0 && P==0)
    { vmessage("error (%s): To compute K, asini *and* P have to be specified.", _function_name());
      return;
    }

  variable M;
  if(qualifier_exists("T0") || qualifier_exists("T90"))
  {
    if(P==0)
    { vmessage("error (%s): In order to use the qualifiers 'T0' or 'T90', 'P' has to be specified as well.", _function_name());
      return;
    }
    variable T0 = qualifier("T0");
    if(qualifier_exists("T90"))  T0 = qualifier("T90") + (omega - PI/2.)/(2*PI) * P;
    M = 2*PI * (x - T0)/P;
  }
  else
    M = 2*PI * x;

  if(asini!=0)
    K = 2*PI * asini / (P*86400*sqrt((1.-e)*(1.+e)));

  variable eccanom = KeplerEquation(M, e);  % eccentric anomaly from Kepler's Equation
  variable theta = 2.*atan( sqrt( (1.+e)/(1.-e) )*tan(eccanom/2.) );  % true anomaly from eccentric anomaly
  return v0 + K * (cos(theta+omega) + e*cos(omega));  % velocity amplitude
}
