define vacuum_to_air()
%!%+
%\function{vacuum_to_air}
%\synopsis{Convert vacuum wavelengths to air wavelengths}
%\usage{Double_Type l_a[] = vacuum_to_air(Double_Type l_v[])}
%\description
%    Convert vacuum wavelengths l_v (in Angstroem) to air wavelengths l_a (in Angstroem)
%    for dry air at 15 degrees Celsius, 101.325 Pa, and with 450 parts per million CO2
%    content. Valid in the wavelength range between 2300 to 17000 Angstroem.
%\notes
%    According to Equation 1 in Ciddor 1996, Applied Optics, 35, 1566:
%       1e8*(l_v-l_a)/l_a = n-1 = k1/(k0-l_v^(-2)) + k3/(k2-l_v^(-2))
%    => l_a = l_v / ( k1*1e-8/(k0-l_v^(-2)) + k3*1e-8/(k2-l_v^(-2)) + 1 )
%       with k0 = 238.0185 * 1e-8 / Angstroem^2,
%            k1 = 5792105  * 1e-8 / Angstroem^2,
%            k2 = 57.362   * 1e-8 / Angstroem^2,
%            k3 = 167917   * 1e-8 / Angstroem^2.
%\qualifiers{
%\qualifier{verbose [=1]}{: Set to 0 to suppress warnings.}
%}
%\example
%    l_a = vacuum_to_air([2000,4000,8000,16000]);
%\seealso{air_to_vacuum}
%!%-
{
  if(_NARGS!=1)
  {
    _pop_n(_NARGS); % remove function arguments from stack
    help(_function_name());
    throw UsageError, sprintf("Usage error in '%s': wrong number of arguments.", _function_name());
  }
  variable verbose = qualifier("verbose", 1);
  variable l_v = ();
  l_v = [l_v]; % cast to array
  if( min(l_v)<2300 or max(l_v)>17000 )
    if(verbose)
      vmessage("Warning in '%s': Wavelength outside of valid range 2300 < l_v < 17000 Angstroem!", _function_name());
  variable k0 = 2.380185e-06;
  variable k1 = 5.792105e-10;
  variable k2 = 5.7362e-07;
  variable k3 = 1.67917e-11;
  variable divisor = ( k1/(k0-l_v^(-2)) + k3/(k2-l_v^(-2)) + 1. );
  variable lo = 2300.;
  variable hi = 17000.;
  divisor[where(l_v < lo)] = ( k1/(k0-lo^(-2)) + k3/(k2-lo^(-2)) + 1. );
  divisor[where(l_v > hi)] = ( k1/(k0-hi^(-2)) + k3/(k2-hi^(-2)) + 1. );
  return l_v / divisor;
}

define air_to_vacuum()
%!%+
%\function{air_to_vacuum}
%\synopsis{Convert air wavelengths to vacuum wavelengths}
%\usage{Double_Type l_v[] = air_to_vacuum(Double_Type l_a[])}
%\description
%    Convert air wavelengths l_a (in Angstroem) to vacuum wavelengths l_v (in Angstroem)
%    for dry air at 15 degrees Celsius, 101.325 Pa, and with 450 parts per million CO2
%    content. Valid in the wavelength range between 2300 to 17000 Angstroem.
%\notes
%    According to Equation 1 in Ciddor 1996, Applied Optics, 35, 1566:
%       1e8*(l_v-l_a)/l_a = n-1 = k1/(k0-l_v^(-2)) + k3/(k2-l_v^(-2))
%    => l_a = l_v / ( k1*1e-8/(k0-l_v^(-2)) + k3*1e-8/(k2-l_v^(-2)) + 1 )
%       with k0 = 238.0185 * 1e-8 / Angstroem^2,
%            k1 = 5792105  * 1e-8 / Angstroem^2,
%            k2 = 57.362   * 1e-8 / Angstroem^2,
%            k3 = 167917   * 1e-8 / Angstroem^2.
%    This function is not analytically solvable for l_v. To zeroth order,
%    however, l_v ~ l_a so that
%       l_v ~ l_a * ( k1*1e-8/(k0-1/l_a^2) + k3*1e-8/(k2-1/l_a^2) + 1 ).
%\qualifiers{
%\qualifier{verbose [=1]}{: Set to 0 to suppress warnings.}
%}
%\example
%    l_v = [2000,4000,8000,16000];
%    print(l_v - air_to_vacuum(vacuum_to_air(l_v)));
%\seealso{vacuum_to_air}
%!%-
{
  if(_NARGS!=1)
  {
    _pop_n(_NARGS); % remove function arguments from stack
    help(_function_name());
    throw UsageError, sprintf("Usage error in '%s': wrong number of arguments.", _function_name());
  }
  variable verbose = qualifier("verbose", 1);
  variable l_a = ();
  l_a = [l_a]; % cast to array
  if( l_a[0]<2300 or l_a[-1]>17000 )
    if(verbose)
      vmessage("Warning in '%s': Wavelength outside of valid range 2300 < l_a < 17000 Angstroem!", _function_name());
  variable k0 = 2.380185e-06;
  variable k1 = 5.792105e-10;
  variable k2 = 5.7362e-07;
  variable k3 = 1.67917e-11;
  variable factor = ( k1/(k0-l_a^(-2)) + k3/(k2-l_a^(-2)) + 1. );
  variable lo = 2300.;
  variable hi = 17000.;
  factor[where(l_a < lo)] = ( k1/(k0-lo^(-2)) + k3/(k2-lo^(-2)) + 1. );
  factor[where(l_a > hi)] = ( k1/(k0-hi^(-2)) + k3/(k2-hi^(-2)) + 1. );
  return l_a * factor;
}
