private variable vsop87_data;

define vsop87()
%!%+
%\function{vsop87}
%\synopsis{calculates high precision planetary positions using the VSOP87 theory}
%\usage{Struct_Type vsop87(JD,planet;qualifiers)}
%\description
% This routine implements the planetary theory VSOP87 of
% Bretagnon and Francou (1988, A&A 202, 309).
% The routine can return high precision planetary positions for all planets
% for a given (array) of times (in TT).
%
% Depending on the qualifiers the routine can return rectangular coordinates
% (X,Y,Z) or ecliptical coordinates (longitude, latitude, distance) in a
% heliocentric coordinate system either for epoch J2000 or for the epoch
% of date. In addition, elliptical orbital elements and standard orbital
% elements can also be calculated.
%
% By default the routine returns rectangular heliocentric coordinates in a
% structure with elements x, y, z, vx, vy, vz.
% Here x, y, z are in an ecliptical coordinate system for the epoch J2000,
% where the x-axis is in the direction of the vernal equinox, the z-axis
% points to the ecliptical North pole, and the y-axis forms a right handed
% coordinate system with the x- and z-axes. The default units are AU and AU/day.
%
% For heliocentric coordinates the planet can have the values
% "Mercury", "Venus", "Earth", "Mars", "Jupiter", "Saturn",
% "Uranus", and "Neptune". For heliocentric rectangular coordinates
% the value "Earth-Moon" will return the position of the
% Earth-Moon barycenter.
%
% In the case of solar system barycentric data, it is not possible to
% obtain the elements of the Earth-Moon barycenter, only the coordinates
% of the Earth can be queried. In addition, for this set of
% coordinates it is also possible to request the rectangular
% position of the Sun.
%
% For the years 1900-2100 interval the precision of the positions
% is at the level of a few milli-arcseconds, with the exception of
% Saturn, for which a precision of "only" 0.1'' is reached.
%
% The precision is better than 1" for the following time intervals
% around AD2000:
% Mercury, Venus, Earth-Moon Barycenter, Mars: 4000 years
% Jupiter, Saturn: 2000 years
% Uranus, Neptune: 6000 years
%
% The time argument to the routine is the Julian Date in Terrestrial Time (TT),
% which corresponds to the international atomic time TAI+32.184s.
%
% This is a powerful routine with a large number of options. A set of helper
% functions provide simpler interfaces.
%
%\qualifiers{
% \qualifier{mjd}{The argument is in MJD(TT), not in JD(TT)}
% \qualifier{tdb}{The argument is in TDB, not TT}
% \qualifier{barycentric}{Return coordinates with respect to the solar system barycenter rather
%                    than heliocentric coordinates. This implies that coordinates are for J2000.}
% \qualifier{spherical}{Return spherical coordinates in the ecliptical coordinate system.
%                       Only possible for heliocentric data.
%                       In this case the structure returned contains the elements
%                       lat, lon, r, latdot, londot, rdot, where lat and lon are the ecliptical
%                       latitude and longitude (in rad), r is the distance in AU, and
%                       latdot, londot, rdot are their time derivatives (in rad/day or
%                       AU/day, respectively.)}
% \qualifier{mean_equinox}{Coordinates are given in the frame defined by the
%                   mean equinox and ecliptic of the date. In VSOP87, this
%                   system is derived from the J2000 one by pure precessional
%                   motion. The default is to return J2000 coordinates.}
% \qualifier{elements}{Return the orbital elements. The structure contains
%                   the following tags:
%                      a: semi-major axis (in AU)
%                      l: mean longitude (rad)
%                      k=e cos(p): elliptical element k
%                      h=e sin(p): elliptical element h
%                      q=sin(g) cos(omega): elliptical element q
%                      p=sin(g) sin(omega): elliptical element p
%                      e: orbital eccentricity
%                      plon: perihelion longitude (in rad)
%                      g: semi inclination (rad)
%                      i: inclination (rad)
%                      omega: longitude of the ascending node (also called G)
%                 }
% \qualifier{deg}{All angle quantities to be returned are in deg, or deg/day.}
% \qualifier{mks}{All distances returned by the function are given in meters or m/s, depending
%                 on the deg qualifier, angles are returned in rad and rad/s or deg and deg/s.}
% \qualifier{cgs}{Dito, but use centimeters instead of meters.}
% \qualifier{vsopfile}{Path to file containing the VSOP quantities; only necessary if
%                 this is a non-standard installation of isisscripts. This qualifier is
%                 only used if the vsop87 data have not yet been read.
%                 By default, the file with the coefficients is searched for in the
%                 directory pointed to by the environment variable ISISSCRIPTS_REFPATH.}
% \qualifier{forcearray}{Force the tags in the returned struct to be arrays. The default is
%                 that they are arrays only if JD is an array.}
%}
%\seealso{planetpos,jpl_eph,jpl_eph_vec}
%!%-
{
variable jdin,planet;
  switch(_NARGS)
  {
   case 2: (jdin,planet) = ();
  }
  { help(_function_name()); return; }

  % protect the JD argument from being changed on the fly
  variable JD=@jdin;
  
  if (qualifier_exists("mjd")) {
     JD=mjd2jd(JD);
  }

  if (qualifier_exists("tdb")) {
      JD=tdb2tt(JD); % use Bretagnon & Francou conversion to TT
  }
  
  if (qualifier_exists("equatorial")) {
     if (qualifier_exists("mean_equinox") or qualifier_exists("spherical")) {
	 throw UsageError,sprintf("%s: Only J2000 allowed for equatorial coordinates\n",_function_name());
     }
  }
    
  % type:
  % 0 -- VSOP87  Heliocentric elliptic elements J2000
  % 1 -- VSOP87A Heliocentric rectangular variables J2000
  % 2 -- VSOP87B Heliocentric spherical variables J2000
  % 3 -- VSOP87C Heliocentric rectangular variables of date
  % 4 -- VSOP87D Heliocentric spherical variables of date
  % 5 -- VSOP87E Barycentric rectangular variables J2000

  variable type=1;
  
  % figure out what theory to use
  if (qualifier_exists("elements")) {
      type=0;
  } else {
      if (qualifier_exists("barycentric")) {
	  type=5;
	  % spherical are handled later
      } else {
	  % heliocentric coordinates
	  if (qualifier_exists("spherical")) {
	      type=2;
	      if (qualifier_exists("mean_equinox")) {
		  type=4;
	      } 
	  } else {
	      % cartesian coordinates
	      type=1;
	      if (qualifier_exists("mean_equinox")) {
		  type=3;
	      }
	  }
      }
  }


  % translation of body code to planet
  variable planets=["","MERCURY","VENUS","EARTH","MARS","JUPITER","SATURN","URANUS","NEPTUNE",""];

  switch(type)
  { case 0: planets[3]="EARTH-MOON";}
  { case 1: planets[9]="EARTH-MOON";}
  { case 5: planets[9]="SUN";}

  variable bcode=where(planets==strup(planet));

  if (length(bcode)==0) {
      variable i;
      variable erstr="";
      variable comma="";
      _for i(1,length(planets)-1,1) {
	  if (planets[i]!="") {
	      erstr+=comma+planets[i];
	  }
	  comma=", ";
      }
      throw UsageError,sprintf("%s: planet qualifier must be ONE of %s\n",_function_name(),erstr);
  }
  bcode=bcode[0];

  %
  % variables of the theory (will be used later as the structure elements)
  % further 
  variable varname;
  variable varnum; % number of variables in the theory
  if (type==0) {
      varnum=6;
      varname={"a","l","k","h","q","p"};
  } else {
      varnum=3;
      if (type==1 or type==3 or type==5) {
	  varname={"x","y","z","vx","vy","vz"};
      } else {
	  varname={"lon","lat","r","londot","latdot","rdot"};
      }
  }
  
  ifnot (__is_initialized(&vsop87_data)) {
      variable searchpath=getenv("ISISSCRIPTS_REFPATH");
      if (searchpath==NULL and qualifier_exists("vsopfile")==0) {
	  throw UsageError,sprintf("%s: vsopfile qualifier not given and ISISSCRIPTS_REFPATH not found\n",_function_name());
      }
      if (searchpath==NULL) {
	  searchpath=".";
      }
      vsop87_data=fits_read_table(qualifier("vsopfile",searchpath+"/vsop87.fits.gz[VSOP87]"));
  }

  variable century=365250.;

  variable T=(JD-2451545.0)/century;

  variable tlen=length(T);

  variable tmpres=Double_Type[tlen,varnum];
  variable derivres=Double_Type[tlen,varnum];

  variable ii,jj;

  _for ii(0,varnum-1,1) {
      variable ndx=where(vsop87_data.type==type and vsop87_data.body==bcode and vsop87_data.ic==ii+1);
      if (length(ndx)>0) {
	  _for jj(0,tlen-1,1) {
	      variable co=cos(vsop87_data.b[ndx]+vsop87_data.c[ndx]*T[jj]);
	      variable ndx1;
	      variable ndx0=where(vsop87_data.alpha[ndx]==0,&ndx1);
	      if (length(ndx0)>0) {
		  tmpres[jj,ii]=sum(vsop87_data.a[ndx[ndx0]]*co[ndx0]);
	      }
	      if (length(ndx1)>0) {
 	          tmpres[jj,ii]+=sum(T[jj]^vsop87_data.alpha[ndx[ndx1]]*vsop87_data.a[ndx[ndx1]]*co[ndx1]);
	      }
	      % derivatives
	      if (type>0) {
		  variable si=sin(vsop87_data.b[ndx]+vsop87_data.c[ndx]*T[jj]);
		  if (length(ndx0)>0) {
		      derivres[jj,ii]=sum(-vsop87_data.a[ndx[ndx0]]*vsop87_data.c[ndx[ndx0]]*si[ndx0]);
		  }
		  if (length(ndx1)>0) {
		      derivres[jj,ii]+=sum(vsop87_data.alpha[ndx[ndx1]]*T[jj]^(vsop87_data.alpha[ndx[ndx1]]-1)*vsop87_data.a[ndx[ndx1]]*co[ndx1]-
		                          T[jj]^vsop87_data.alpha[ndx[ndx1]]*vsop87_data.a[ndx[ndx1]]*vsop87_data.c[ndx[ndx1]]*si[ndx1]);
		  }
	      }
	  }
      }
  }

  % normalize derivative to per day
  if (type>0) {
      derivres=derivres/century;
  }

  % normalize longitudes
  if (type==0 or type==2 or type==4) {
      ndx=(type==0) ? 1 : 0;
      tmpres[*,ndx]=tmpres[*,ndx] mod (2*PI);
      variable nn=where(tmpres[*,ndx]<0);
      if (length(nn)>0) {
	  tmpres[nn,ndx]+=2*PI;
      }
  }

  %

  %
  % generate structure for returning the values
  %

  % missing structure tags for elliptical elements
  if (type==0) {
      list_append(varname,"e");
      list_append(varname,"plon");
      list_append(varname,"g");
      list_append(varname,"i");
      list_append(varname,"omega");
  }
  varname=list_to_array(varname);
  
  % generate the structure
  variable retval=@Struct_Type(varname);
  if (type==0) {
      if (typeof(JD)==Array_Type or qualifier_exists("forcearray")) {
	  _for i(0,5,1) {
	      set_struct_field(retval,varname[i],tmpres[*,i]);
	  }
      } else {
	  _for i(0,5,1) {
	      set_struct_field(retval,varname[i],tmpres[0,i]);
	  }
      }
      retval.plon=atan2(retval.h,retval.k);
      retval.omega=atan2(retval.plon,retval.q);
      % hoping that the cosine isn't 0 somewhere...
      retval.e=retval.k/cos(retval.plon); 
      retval.g=retval.q/cos(retval.omega);
      retval.i=2*retval.g;
  } else {
      if (typeof(JD)==Array_Type or qualifier_exists("forcearray")) {
	  _for i(0,2,1) {
	      set_struct_field(retval,varname[i],tmpres[*,i]);
	      set_struct_field(retval,varname[i+3],derivres[*,i]);
	  }
      } else {
	  _for i(0,2,1) {
	      set_struct_field(retval,varname[i],tmpres[0,i]);
	      set_struct_field(retval,varname[i+3],derivres[0,i]);
	  }
      }
  }

  variable tag;
  if (qualifier_exists("deg")) {
      foreach tag(["l","p","plon","g","i","omega","lat","lon","latdot","londot"]) {
	  if (struct_field_exists(retval,tag)) {
	      set_struct_field(retval,tag,get_struct_field(retval,tag)*180./PI);
	  }
      }
  }
  if (qualifier_exists("mks") or qualifier_exists("cgs")) {
      variable au=149597870691.; % meters, DE405 value, per Kaplan, 2005, USNO Circular 179
      if (qualifier_exists("cgs")) {
	  au*=100.;
      }
      % convert lengths
      foreach tag(["x","y","z","vx","vy","vz","r","rdot","a"]) {
	  if (struct_field_exists(retval,tag)) {
	      set_struct_field(retval,tag,get_struct_field(retval,tag)*au);
	  }
      }
      % convert rates
      foreach tag(["vx","vy","vz","rdot"]) {
	  if (struct_field_exists(retval,tag)) {
	      set_struct_field(retval,tag,get_struct_field(retval,tag)/86400.0);
	  }
      }
  }

  return retval;
}

