% helper function for a fast polynomial
private define tout_poly(a,b,c,d,e,logz) {
    return (((e*logz+d)*logz+c)*logz+b)*logz+a;
}

define zams () 
%
%!%+
%\function{zams}
%\synopsis{return properties of the ZAMS per Tout et al., 1996, MNRAS 281, 257}
%\usage{(radius,luminosity,temperature)=zams(mass;z=metallicity);}
%\qualifiers{
%    \qualifier{z}{metallicity (between 0.0001 and 0.03; default: 0.02 (solar))}
%}
%\description
%  This function uses the rational function fits of Tout et al. (1996, MNRAS 281, 257)
%  and calculates the radius, luminosity, and temperature for zero age main sequence stars
%  of a given mass (in solar units;  can be an array, must be between 0.1 and 100 Msun).
%  The radius and luminosity are returned in solar units, the temperature is in K.
%  Tout et al. state that in general the fits are better than 7.5per cent in luminosity and
%  5 per cent in radius, for solar metallicity they are good to 3 per cent in L and
%  1.2 per cent in R.
%!%-
%  
{
    variable mass;
    switch(_NARGS)
    { case 1: mass=(); }
    { help(_function_name()); return;}

    % metallicity
    variable zz=qualifier("z",0.02); 
    
    % check parameter validity
    if (zz<0.0001 || zz>0.03) {
	vmessage("Metallicity must be between 0.0001 and 0.03");
    }
    variable mmin=min(mass);
    variable mmax=max(mass);

    if (mmin<0.1 || mmin>100. || mmax<0.1 || mmax>100.) {
	vmessage("Mass must be between 0.1 and 100 solar masses");
    }
    
    % variable xx=0.76-3.*zz; % Hydrogen
    % variable yy=0.24+2.*zz; % Helium

    variable zsun=0.02; % solar metallicity
    variable logz=log10(zz/zsun);

    % Tout, Table 1 
    variable alpha  =tout_poly(0.39704170, -0.32913574, 0.34776688, 0.37470851,0.09011915,logz);
    variable beta   =tout_poly(8.52762600,-24.41225973,56.43597107,37.06152575,5.45624060,logz);
    variable gamma  =tout_poly(0.00025546, -0.00123461,-0.00023246, 0.00045519,0.00016176,logz);
    variable deltaT =tout_poly(5.43288900, -8.62157806,13.44202049,14.51584135,3.39793084,logz);
    variable epsilon=tout_poly(5.56357900,-10.32345224,19.44322980,18.97361347,4.16903097,logz);
    variable zeta   =tout_poly(0.78866060, -2.90870942, 6.54713531, 4.05606657,0.53287322,logz);
    variable eta    =tout_poly(0.00586685, -0.01704237, 0.03872348, 0.02570041,0.00383376,logz);

    % Tout, Table 2
    variable theta  =tout_poly( 1.71535900,  0.62246212, -0.92557761, -1.16996966,-0.30631491,logz);
    variable iota   =tout_poly( 6.59778800, -0.42450044,-12.13339427,-10.73509484,-2.51487077,logz);
    variable kappa  =tout_poly(10.08855000, -7.11727086,-31.67119479,-24.24848322,-5.33608972,logz);
    variable lambda =tout_poly( 1.01249500,  0.32699690, -0.00923418, -0.03876858,-0.00412750,logz);
    variable mu     =tout_poly( 0.07490166,  0.02410413,  0.07233664,  0.03040467, 0.00197741,logz);
    variable nu     =tout_poly( 0.01077422,  0.00000000,  0.00000000,  0.00000000, 0.00000000,logz);
    variable xi     =tout_poly( 3.08223400,  0.94472050, -2.15200882, -2.49219496,-0.63848738,logz);
    variable omikron=tout_poly(17.84778000, -7.45345690,-48.96066856,-40.05386135,-9.09331816,logz);
    variable pi     =tout_poly( 0.00022582, -0.00186899,  0.00388783,  0.00142402,-0.00007671,logz);


    % Tout, Eq. (1)
    % should probably implement parts here using a horner scheme
    variable lum=(alpha*mass^5.5 + beta*mass^11.)/(gamma+mass^3+deltaT*mass^5+epsilon*mass^7+zeta*mass^8+eta*mass^9.5);

    % Tout, Eq. (2)
    % dito
    variable radius=(theta*mass^2.5+iota*mass^6.5+kappa*mass^11.+lambda*mass^19.+mu*mass^19.5)/(nu+xi*mass^2.0+omikron*mass^8.5+mass^18.5+pi*mass^19.5);
    
    % L = 4pi r^2 sigma t^4
    variable lsun=3.828e26; % in Watt(urgh)
    variable sigma_sb=5.670367e-8; % w/m^2 k^4 (urgh)
    variable rsun=695500e3; % m
    variable temperature=( lsun*lum/(4*PI*(rsun*radius)^2.*sigma_sb) )^(1./4.);
    
    return(radius,lum,temperature);
}
