define EasterSunday()
%!%+
%\function{EasterSunday}
%\synopsis{Calculate the JD or date of Easter Sunday for the Gregorian or Julian calendar}
%\usage{jd=EasterSunday(year)}
%\description
%This function calculates the JD of Easter Sunday for the Gregorian or
%Julian (=orthodox) calendar using the algorithms given by J. Meeus, 1998,
%Astronomical Algorithms, 2nd ed, William-Bell.
%
%The function either returns the JD (more precise: the JD at midnight
%of Easter Sunday), or a struct with the same format as DateOfJD.
%
%This function is array safe, i.e., an array of years can be given.
%
%\qualifiers{
%\qualifier{mjd}{Return dates in  MJD, not in JD.}
%\qualifier{orthodox}{Return the JD of Easter Sunday using the rules
%                  appropriate for Eastern Christianity such as the
%                  Russian Orthodox church.}
%\qualifier{get_date}{Return date as a struct (year, month, date, hour,
%                  minute, second). If the orthodox keyword is given,
%                  the date will be in the Julian calendar, otherwise
%                  it will be in the Gregorian calendar.}
%\qualifier{backward}{An undocumented function with the same name
%                  as this one existed in the isisscripts until
%                  17 January 2020. It returned the date of Easter Sunday
%                  in the Gregorian calendar as the number of days
%                  since -1 March (sic!) of the given year. This
%                  qualifier switches the current function to this
%                  backward compatible behavior.}
%}
%\seealso{DateOfJD}
%!%-
{
    variable year=();

    variable x=int(year);

    variable a,b,c,d,e,f,g,h,i,k,l,m,n,p,t,tt;
    
    if (qualifier_exists("orthodox")) {
	% Meeus, p.69
	a=x mod 4;
	b=x mod 7;
	c=x mod 19;
	d=(19*c+15) mod 30;
	e=(2*a+4*b-d+34) mod 7;
	tt=(d+e+114);
	n=tt/31; % month
	p=1+(tt mod 31); % day
    } else {
	% Meeus, p. 67
	a=x mod 19;
	b=x/100;
	c=x mod 100;
	d=b/4;
	e=b mod 4;
	f=(b+8)/25;
	g=(b-f+1)/3;
	h=(19*a+b-d-g+15) mod 30;
	i=c/4;
	k=c mod 4;
	l=(32+2*e+2*i-h-k) mod 7;
	m=(a+11*h+22*l)/451;
	tt=h+l-7*m+114;
	n=tt/31;    % month
	p=1+(tt mod 31); % day
	if (qualifier_exists("backward")) {
	    variable jd=JDofDate(x,n,p);
	    variable jd0=JDofDate(x,3,-1);
	    return jd-jd0;
	}
    }

    if (qualifier_exists("get_date")) {
	if (typeof(x)==Integer_Type) {
	    return struct{year=x,month=n,day=p,hour=0,minute=0,second=0.};
	} else {
	    variable hh=Integer_Type[length(x)];
	    variable mm=Integer_Type[length(x)];
	    variable ss=Double_Type[length(x)];
	    return struct{year=x,month=n,day=p,hour=hh,minute=mm,second=ss};
	}
    }

    if (qualifier_exists("orthodox")) {
	if (qualifier_exists("mjd")) {
	    return JDofDate(x,n,p;julian_calendar,mjd);
	}
	return JDofDate(x,n,p;julian_calendar);
    }

    if (qualifier_exists("mjd")) {
	return JDofDate(x,n,p;mjd);
    }
    return JDofDate(x,n,p);
}
