define equation_equinoxes() {
%!%+
%\function{equation_equinoxes}
%\usage { egam = equation_equinoxes(JD;qualifiers)}
%\qualifiers{
%   \qualifier{mjd}{the time argument is in MJD, not in JD}
%   \qualifier{deg}{return equation of equinoxes in degrees}
%   \qualifier{arcsec}{return equation of equinoxes in arcseconds}
%   \qualifier{mas}{return equation of equinoxes in milliarcseconds}
%   \qualifier{seconds}{return equation of equinoxes in seconds}
%}
%
%\description
% This function calculates the equation of equinoxes, i.e., the 
% difference between the Greenwich Mean Siderial Time and the
% Greenwich Apparent Siderial Time caused by the motion of the
% equinox due to nutation. The implementation uses a truncated
% form of a longer series due to the IERS (2003), given by
% Eq. 2.14 of Kaplan (2009, USNO circular 181). The default
% returns the equation of equinoxes in radian (more commonly
% the values are tabulated in units of seconds).
%
% This function is array safe.
%
%\seealso{gmst,nutation_angles}
%!%-
variable JD;
    switch(_NARGS)
    { case 1 : JD=(); }
    { help(_function_name()); return;}

    variable T;
    if (qualifier_exists("mjd")) {
	T=((JD-51544.5)+qualifier("JD_frac",0.))/36525.0;
    } else {
	T=((JD-2451545.)+qualifier("JD_frac",0.))/36525.0;
    }

    % conversion parameters
    variable asec360=3600.*360.;
    variable asec2rad=PI/648000.;

    % Fundamental arguments from the Simon et al. (1994) lunar theory
    % see nutation_angles.sl
    
    
    variable Om, F, D;
    
    % Mean argument of the latitude of the Moon
    F = ((335779.526232 +
               T * (1739527262.8478 +
               T * (      - 12.7512 +
               T * (      -  0.001037 +
               T * (         0.00000417))))) mod asec360) * asec2rad;

    % Mean elongation of the moon from the Sun
    D = ((1072260.70369 +
               T * (1602961601.2090 +
               T * (       - 6.3706 +
               T * (         0.006593 +
               T * (       - 0.00003169))))) mod asec360) * asec2rad;

    % Mean longitude of the Moon's ascending node
    Om= ((450160.398036 +
               T * ( - 6962890.5431 +
               T * (         7.4722 +
               T * (         0.007702 +
               T * (       - 0.00005939))))) mod asec360) * asec2rad;

    variable eps,dpsi,deps;
    if (qualifier_exists("mjd")) {
        eps=obliquity(JD;mjd); % in rad
        (dpsi,deps)=nutation_angles(JD;mjd,arcsec); % in arcsec!
    } else {
        eps=obliquity(JD); % in rad
        (dpsi,deps)=nutation_angles(JD;arcsec); % in arcsec!
    }
        
    variable egam=dpsi*cos(eps)
       + 0.00264096 * sin(Om)
       + 0.00006352 * sin(2.*Om)
       + 0.00001175 * sin(2.*F-2.*D+3.*Om)
       + 0.00001121 * sin(2.*F-2.*D+Om)
       - 0.00000455 * sin(2.*F-2.*D+2.*Om)
       + 0.00000202 * sin(2.*F+3.*Om)
       + 0.00000198 * sin(2.*F+Om)
       - 0.00000172 * sin(3.*Om)
       - 0.00000087 * T * sin(Om);

    if (qualifier_exists("arcsec")) {
        return egam;
    } 
    if (qualifier_exists("deg")) {
        return egam/3600.;
    }
    if (qualifier_exists("mas")) {
        return egam*1000.;
    }
    if (qualifier_exists("seconds")) {
        return egam/15.;
    }
    return egam*asec2rad;
}
