define tt2tcg()
%!%+
%\function{tt2tcg}
%\synopsis{Convert a (M)JD in Terrestrial Time (TT, aka TDT or ET) into TCG}
%\usage{tai=tt2tcg(JD)}
%\description
%  This routine converts a Julian Date that is given in Terrestrial Time
%  (formerly known as Terrestrial Dynamic Time or Ephemeris Time) into
%  geocentric coordinate time (TCG). The two time scales differ by a constant
%  rate(!) because TT uses the SI second at the surface of the geoid, and TCG uses
%  the SI second at the potential of the center of the Earth.
%  This routine is array safe.
%\qualifiers{
%\qualifier{mjd}{The argument is given in MJD, not in JD.}
%\qualifier{deltat}{Return the TCG-TT in seconds, rather than the corrected (M)JD.}
%}
%\seealso{tcg2tt,utc2tai,tai2utc,tai2tt,tt2tdb}
%!%-
{
    variable JD;
    switch(_NARGS)
    {
	case 1: JD=();
    }
    {
	help(_function_name()); return;
    }

    % rate factor (exact by definition)
    variable lg=6.969290134e-10;
    
%    variable diff=-32.184;

    % zero point of TT, TCG, and TCB
    % (1977 Jan 1 00:00:00 TAI)
    variable t0;
    if (qualifier_exists("mjd")) {
        t0=43144.0+32.184/86400.; % TT
    } else {
        t0=2443144.5+32.184/86400.; % TT
    }
    % Kaplan, eq 2.3
    % TT= TCG-lg*(TCG-t0);
    % TT = TCG - lg*TCG + lg*t0
    % TT = TCG*(1-lg) + lg*t0
    % TT - lg*t0 = TCG*(1-lg)
    % (TT-lg*t0)/(1-lg) = TCG

    variable tcg=(JD-lg*t0)/(1.-lg);

    if (qualifier_exists("deltat")) {
	return (tcg-JD)*86400.;
    }
    return tcg;
}

define tcg2tt()
%!%+
%\function{tcg2tt}
%\synopsis{Convert a (M)JD in TCG to Terrestrial Time (TT, aka TDT or ET)}
%\usage{tai=tcg2tt(JD)}
%\description
%  This routine converts a Julian Date that is given in geocentric coordinate
%  time (TCG) to Terrestrial Time. The two time scales differ by a constant
%  rate(!) because TT uses the SI second at the surface of the geoid, and TCG
%  uses the SI second at the potential of the center of the Earth.
%  This routine is array safe.
%\qualifiers{
%\qualifier{mjd}{The argument is given in MJD, not in JD.}
%\qualifier{deltat}{Return the TT-TCG in seconds, rather than the corrected (M)JD.}
%}
%\seealso{tcg2tt,utc2tai,tai2utc,tai2tt,tt2tdb}
%!%-
{
    variable JD;
    switch(_NARGS)
    {
	case 1: JD=();
    }
    {
	help(_function_name()); return;
    }

    % rate factor (exact by definition)
    variable lg=6.969290134e-10;
    
    % zero point of TT, TCG, and TCB
    % (1977 Jan 1 00:00:00 TAI)
    variable t0;
    if (qualifier_exists("mjd")) {
        t0=43144.0+32.184/86400.; % TT
    } else {
        t0=2443144.5+32.184/86400.; % TT
    }
    % Kaplan, eq 2.3
    % TT= TCG-lg*(TCG-t0);
    % TT = TCG - lg*TCG + lg*t0
    % TT = TCG*(1-lg) + lg*t0
    % TT - lg*t0 = TCG*(1-lg)
    % (TT-lg*t0)/(1-lg) = TCG

    variable tt=JD-lg*(JD-t0);

    if (qualifier_exists("deltat")) {
	return (tt-JD)*86400.;
    }
    return tt;
}

