private variable morrison_deltat_y=[
-500.,-400.,-300.,-200.,-100.,0.,100.,200.,300.,400.,500.,600.,700.,800.,
900.,1000.,1100.,1200.,1300.,1400.,1500.,1600.,1700.,1710.,1720.,1730.,1740.,
1750.,1760.,1770.,1780.,1790.,1800.,1810.,1820.,1830.,1840.,1850.,1860.,
1870.,1880.,1890.,1900.,1910.,1920.,1930.,1940.,1950.,1960.,1970.,1980.,1990.,2000.];

private variable morrison_deltat_d=[
17190.,15530.,14080.,12790.,11640.,10580.,9600.,8640.,7680.,6700.,
5710.,4740.,3810.,2960.,2200.,1570.,1090.,740.,490.,320.,200.,120.,9.,
10.,11.,11.,12.,13.,15.,16.,17.,17.,14.,13.,12.,8.,6.,7.,8.,2.,-5.,
-6.,-3.,10.,21.,24.,24.,29.,33.,40.,51.,57.,65.];


define ut12tt() {
%!%+
%\function{ut12tt}
%\synopsis{Convert a (M)JD in UT1 into TT}
%\usage{utc=ut12tt(JD)}
%
%\qualifiers{
%\qualifier{mjd}{The argument is given in MJD, not in JD.}
%\qualifier{deltat}{Return the TT-UT1 in seconds, rather than the corrected (M)JD.}
%\qualifier{morrison}{Perform the correction using a linear interpolation
%                       to the points by Morrison and Stephenson, 2004,
%                       Journal for the History of Astronomy 35, 327.
%                       Not really good after 2000}
%}
%\description
% This function converts a (Modified) Julian Date that is given in UT1,
% i.e., the timescale that defines the earth rotation angle,
% into a (M)JD in TT.
%
% By default, for years before 2005 the function uses the polynomial
% expressions for TT-UT1 published by F. Espenak and J. Meeus, 2006,
% Five Millenium Canon of Solar Eclipses: -1999 to +3000,
% NASA TP-2006-214141, for years after 2005 the polynomials published
% by Espenak at http://www.eclipsewise.com/help/deltatpoly2014.html are
% used.
%
% Alternatively, a linar interpolation in the solar eclipse derived
% data by Morrison and Stephenson (2000) can be used (see morrison
% qualifier).
%
% The uncertainty of the result is 1s or less from 1780 until 1830,
% less than 1s from 1830-2000. It increases quadratically backwards
% from 1820, sigma ~32((year-1820)/100)^2.
%
% This function is array safe.
%\seealso{tt2ut1,utc2tai,tai2utc,tai2tt}
%!%-
    variable JD;
    switch(_NARGS)
    {
	case 1: JD=();
    }
    {
	help(_function_name()); return;
    }

    variable year;
    if (qualifier_exists("mjd")) {
	year=EpochofJD(JD;mjd);
    } else {
	year=EpochofJD(JD);
    }

    variable deltat;
    
    if (qualifier_exists("morrison")) {
	% TT-UT1 in seconds	
	deltat=interpol(year,morrison_deltat_y,morrison_deltat_d);

	if (qualifier_exists("deltat")) {
	    return deltat;
	}
	return JD+deltat/86400.;
    }


    deltat=Double_Type[length(year)];
    variable u,tmp;
    variable ndx=where(year<-500.);
    if (length(ndx)>0) {
	u=(year[ndx]-1820.)/100.;
	deltat[ndx]=-20.+32.*u*u;
    }
    ndx=where(-500<=year<500.);
    if (length(ndx)>0) {
	u=year[ndx]/100.;
	deltat[ndx]=(((((0.0090316521*u+0.022174192)*u-0.1798452)*u-5.952053)*u+33.78311)*u-1014.41)*u+10583.6;
    }
    ndx=where(+500<=year<1600);
    if (length(ndx)>0) {
	u=(year[ndx]-1000.)/100.;
	deltat[ndx]=(((((0.0083572073*u-0.005050998)*u-0.8503463)*u+0.319781)*u+71.23472)*u-556.01)*u+1574.2;
    }
    ndx=where(+1600<=year<1700);
    if (length(ndx)>0) {
	u=year[ndx]-1600.;
	deltat[ndx]=((1./7129.*u-0.01532)*u-0.9808)*u+120.;
    }
    ndx=where(+1700<=year<1800.);
    if (length(ndx)>0) {
	u=year[ndx]-1700.;
	deltat[ndx]=(((-1./1174000.*u+0.00013336)*u-0.0059285)*u+0.1603)*u+8.83;
    }
    ndx=where(1800<=year<1860.);
    if (length(ndx)>0) {
	u=year[ndx]-1800.;
	deltat[ndx]=((((((0.000000000875*u-0.0000001699)*u+0.0000121272)*u-0.00037436)*u+0.0041116)*u+0.0068612)*u-0.332447)*u+13.72;
    }
    ndx=where(1860.<=year<1900.);
    if (length(ndx)>0) {
	u=year[ndx]-1860.;
	deltat[ndx]=((((1./233174.*u-0.0004473624)*u+0.01680668)*u-0.251754)*u+0.5737)*u+7.62;
    }
    ndx=where(1900.<=year<1920.);
    if (length(ndx)>0) {
	u=year[ndx]-1900.;
	deltat[ndx]=(((-0.000197*u+0.0061966)*u-0.0598939)*u+1.494119)*u-2.79;
    }
    ndx=where(1920.<=year<1941.);
    if (length(ndx)>0) {
	u=year[ndx]-1920.;
	deltat[ndx]=((0.0020936*u-0.076100)*u+0.84493)*u+21.20;
    }
    ndx=where(1941<=year<1961);
    if (length(ndx)>0) {
	u=year[ndx]-1950.;
	deltat[ndx]=((1./2547*u-1./233)*u+0.407)*u+29.07;
    }
    ndx=where(1961<=year<1986);
    if (length(ndx)>0) {
	u=year[ndx]-1975.;
	deltat[ndx]=((-1./718*u-1./260.)*u+1.067)*u+45.45;
    }
    ndx=where(1986<=year<2005.);
    if (length(ndx)>0) {
	u=year[ndx]-2000.;
	deltat[ndx]=((((0.00002373599*u+0.000651814)*u+0.0017275)*u-0.060374)*u+0.3345)*u+63.86;
    }
    ndx=where(2005.<=year<2015.);
    if (length(ndx)>0) {
	u=year[ndx]-2005.;
	deltat[ndx]=64.69+0.2930*u ;
	
    }
    ndx=where(2015.<=year);
    if (length(ndx)>0) {
	u=year[ndx]-2015.;
	deltat[ndx]=(0.0039755*u+0.3645)*u+67.62;
    }

    % ensure correct return type
    if (typeof(JD)!=Array_Type) {
	deltat=deltat[0];
    }

    if (qualifier_exists("deltat")) {
        % deltat is TT-UT1
        % minus sign: we want to be consistent with the other
        % conversion routines, so need to return UT1-TT
	return -deltat;
    }
    % deltat=TT-UT1 -> TT = deltat+UT1
    return JD+deltat/86400.;
}

define tt2ut1() {
%!%+
%\function{tt2ut1}
%\synopsis{Convert a (M)JD in TT into UT1}
%\usage{ut1=tt2ut1(JD)}
%\qualifiers{
%\qualifier{mjd}{The argument is given in MJD, not in JD.}
%\qualifier{deltat}{Return the TT-UT1 in seconds, rather than the corrected (M)JD.}
%\qualifier{morrison}{Perform the correction using a linear interpolation
%                       to the points by Morrison and Stephenson, 2004,
%                       Journal for the History of Astronomy 35, 327.
%                       Not good after 2000.}
%}
%\description
%
% This function converts a (Modified) Julian Date that is given in
% TT into UT1, i.e., the timescale that defines the earth rotation angle.
%
% By default, for years before 2005 the function uses the polynomial
% expressions for TT-UT1 published by F. Espenak and J. Meeus, 2006,
% Five Millenium Canon of Solar Eclipses: -1999 to +3000,
% NASA TP-2006-214141, for years after 2005 the polynomials published
% by Espenak at http://www.eclipsewise.com/help/deltatpoly2014.html are
% used.
%
% Alternatively, a linar interpolation in the solar eclipse derived
% data by Morrison and Stephenson (2000) can be used (see morrison
% qualifier).
%
% The uncertainty of the result is 1s or less from 1780 until 1830,
% less than 1s from 1830-2000. It increases quadratically backwards
% from 1820, sigma ~32((year-1820)/100)^2.
%
% This routine is array safe.
%
% Note: Because of the uncertainty in TT-UT1, the same
% polynomial is used for the TT->UT1 and UT1->TT conversion,
% and therefore it generally is NOT true that JD==ut12tt(tt2ut1(JD)).
%
%\seealso{ut12tt,utc2tai,tai2utc,tai2tt}
%!%-
    variable JD;
    switch(_NARGS)
    {
	case 1: JD=();
    }
    {
	help(_function_name()); return;
    }

    % obtain UT1-TT
    variable deltat;
    if (qualifier_exists("mjd")) {
	deltat=ut12tt(JD;mjd,deltat);
    } else {
	deltat=ut12tt(JD;deltat);
    }
    
    if (qualifier_exists("deltat")) {
        % return TT-UT1
	return -deltat;
    }
    
    return JD+deltat/86400.;
}
