private variable tai2utc_data;

private define read_leapfile(leapfile) {
    % helper function to read a leapfile
    % the argument is the file to read.
    
    variable statret=stat_file(leapfile);
    if (statret==NULL) {
	throw IOError,sprintf("%s: problem with leapfile %s or leapfile not found: Errno %i: %s\n",
	                      _function_name(),leapfile,errno,errno_string(errno));
    }
    variable refjd={};
    variable diffsec={};
    variable mjdoff={};
    variable jdoff={};
    variable rate={};
    % the leapfile is a fixed format file, it is more efficient to read it
    % directly rather than using the readascii function
    variable leap=fopen(leapfile,"r");
    if (leap==NULL) {
	throw IOError, sprintf("%s: error opening leapfie %s: Errno %i: %s\n",
	                       _function_name(),leapfile,errno,errno_string(errno));
    }
    while (not feof(leap)) {
	variable line;
	variable num=fgets(&line,leap);
	if (num>0) {
	    list_append(refjd,atof(substr(line,18,9)));
	    list_append(diffsec,atof(substr(line,37,12)));
	    list_append(mjdoff,atof(substr(line,60,7)));
	    list_append(rate,atof(substr(line,70,10)));
	}
    }
    ()=fclose(leap);
    mjdoff=list_to_array(mjdoff),
    refjd=list_to_array(refjd),
    tai2utc_data=struct{
	refjd=refjd,
	refmjd=jd2mjd(refjd),
	diffsec=list_to_array(diffsec),
	mjdoff=mjdoff,
	jdoff=mjd2jd(mjdoff),
	rate=list_to_array(rate)
    };
}

define utc2tai()
%!%+
%\function{utc2tai}
%\synopsis{Convert a (M)JD in civil time (UTC) into International Atomic Time}
%\usage{tai=utc2tai(JD)}
%\description
%This routine converts a Julian Date that is in civil time (Coordinated
%Universal Time, UTC) into International Atomic Time (TAI) by applying
%the relevant correction for leap seconds. This routine is array safe.
%\qualifiers{
%\qualifier{mjd}{The argument is given in MJD, not in JD.}
%\qualifier{deltat}{Return the TAI-UTC in seconds, rather than the corrected (M)JD.}
%\qualifier{leapfile}{path to a file containing the leap second
%          information. This file is available, e.g., from
%          ftp://maia.usno.navy.mil/ser7/tai-utc.dat
%          Most astronomical software systems distribute this file. The default
%          is to look for it in getenv("LHEASOFT")/refdata}
%}
%\seealso{utc2tai,tt2tai,tai2tt}
%!%-
{
    variable JD;
    switch(_NARGS)
    {
	case 1: JD=();
    }
    {
	help(_function_name()); return;
    }

    if ((not __is_initialized(&tai2utc_data)) or qualifier_exists("leapfile")) {
	read_leapfile(qualifier("leapfile",getenv("LHEASOFT")+"/refdata/tai-utc.dat"));
    }

    variable ndx;
    variable diff={};
    variable jj;

    if (qualifier_exists("mjd")) {
	foreach jj (JD) {
	    if (jj<tai2utc_data.refmjd[0]) {
		throw UsageError,sprintf("%s: TAI is only defined after MJD %15.1f\n",
		                         _function_name(),tai2utc_data.refmjd[0]);
	    }
	    ndx=(where(tai2utc_data.refmjd<=jj))[-1];
	    list_append(diff,tai2utc_data.diffsec[ndx]+ (jj-tai2utc_data.mjdoff[ndx])*tai2utc_data.rate[ndx]);
	}
    } else {
	foreach jj (JD) {
	    if (jj<tai2utc_data.refjd[0]) {
		throw UsageError,sprintf("%s: TAI is only defined after JD %15.1f\n",
		                         _function_name(),tai2utc_data.refjd[0]);
	    }
	    ndx=(where(tai2utc_data.refjd<=jj))[-1];
	    list_append(diff,tai2utc_data.diffsec[ndx]+ (jj-tai2utc_data.jdoff[ndx])*tai2utc_data.rate[ndx]);
	}
    }

    if (typeof(JD)==Array_Type) {
	diff=list_to_array(diff);
    } else {
	diff=diff[0];
    }

    if (qualifier_exists("deltat")) {
	return diff;
    }
    return JD+diff/86400.0;
}

define tai2utc()
%!%+
%\function{tai2utc}
%\synopsis{Converts a (M)JD in International Atomic Time into civil time (UTC)}
%\usage{utc=tai2utc(JD)}
%\description
%This routine converts a Julian Date that is given in international
%atomic time into Coordinated Universal Time (UTC) by applying the
%relevant correction for leap seconds. This routine is array safe.
%
%NOTE: This function is only approximately correct. Within about a
%minute of a leap second this routine can be off by as much as
%one second.
%\qualifiers{
%\qualifier{mjd}{The argument is given in MJD, not in JD.}
%\qualifier{deltat}{Return the UTC-TAI in seconds, rather than the corrected (M)JD.}
%\qualifier{leapfile}{Path to a file containing the leap second
%          information. This file is available, e.g., from
%          ftp://maia.usno.navy.mil/ser7/tai-utc.dat
%          Most astronomical software systems distribute this file. The default
%          is getenv("LHEASOFT")/refdata}
%}
%\seealso{utc2tai,tt2tai,tai2tt}
%!%-
{
    variable JD;
    switch(_NARGS)
    {
	case 1: JD=();
    }
    {
	help(_function_name()); return;
    }

    if ((not __is_initialized(&tai2utc_data)) or qualifier_exists("leapfile")) {
	read_leapfile(qualifier("leapfile",getenv("LHEASOFT")+"/refdata/tai-utc.dat"));
    }

    variable diff;
    if(qualifier_exists("mjd")) {
	diff=utc2tai(JD;mjd,deltat);
    } else {
	diff=utc2tai(JD;deltat);
    }

    if (qualifier_exists("deltat")) {
      return -1.*diff;
    }

    % FIXME: the result is wrong if we are close to a leap second
    return JD-diff/86400.;
}
